﻿/**
History:
	$(HIST_VER 1.120,
		$(HIST_NEW)
	)
*/
module etc.negui.file._hide.file;

debug import std.stdio: wl = writefln;
debug(file) void main() {}


private enum {
/+
	CSIDL_DESKTOP                 = 0x0000, // 「デスクトップ」（ネームスペースのルートを表す仮想フォルダ）
	CSIDL_INTERNET                = 0x0001, // 「Internet Explorer」（仮想フォルダ）
	CSIDL_PROGRAMS                = 0x0002, // 「プログラム」（ファイルシステムディレクトリ）
	CSIDL_CONTROLS                = 0x0003, // 「コントロール パネル」（仮想フォルダ）
	CSIDL_PRINTERS                = 0x0004, // 「プリンタ」（仮想フォルダ）
	CSIDL_PERSONAL                = 0x0005, // 「マイ ドキュメント」（ファイルシステムディレクトリ）
	CSIDL_FAVORITES               = 0x0006, // 「お気に入り」（ファイルシステムディレクトリ）
	CSIDL_STARTUP                 = 0x0007, // 「スタートアップ」（ファイルシステムディレクトリ）
	CSIDL_RECENT                  = 0x0008, // 「最近使ったファイル」（ファイルシステムディレクトリ）
	CSIDL_SENDTO                  = 0x0009, // 「SendTo」（ファイルシステムディレクトリ）
	CSIDL_BITBUCKET               = 0x000a, // 「ごみ箱」（仮想フォルダ）
	CSIDL_STARTMENU               = 0x000b, // 「スタートメニュー」（ファイルシステムディレクトリ）
	CSIDL_MYMUSIC                 = 0x000d, // 「マイ ミュージック」（ファイルシステムディレクトリ）
	CSIDL_MYVIDEO                 = 0x000e, // 
	CSIDL_DESKTOPDIRECTORY        = 0x0010, // 「デスクトップ」上のファイルオブジェクトを格納するフォルダ（ファイルシステムディレクトリ）
	CSIDL_DRIVES                  = 0x0011, // 「マイ コンピュータ」（仮想フォルダ）
	CSIDL_NETWORK                 = 0x0012, // 「ネットワークコンピュータ」（仮想フォルダ）
	CSIDL_NETHOOD                 = 0x0013, // 「NetHood」（ファイルシステムディレクトリ）
	CSIDL_FONTS                   = 0x0014, // 「Fonts」（フォントを含む仮想フォルダ）
	CSIDL_TEMPLATES               = 0x0015, // ドキュメントテンプレートが格納されるフォルダ（ファイルシステムディレクトリ）
	CSIDL_COMMON_STARTMENU        = 0x0016, // AllUsers の「スタートメニュー」（ファイルシステムディレクトリ）
	CSIDL_COMMON_PROGRAMS         = 0x0017, // AllUsers の「プログラム」（ファイルシステムディレクトリ）
	CSIDL_COMMON_STARTUP          = 0x0018, // AllUsers の「スタートアップ」（ファイルシステムディレクトリ）
	CSIDL_COMMON_DESKTOPDIRECTORY = 0x0019, // AllUsers の「デスクトップ」（ファイルシステムディレクトリ）
	CSIDL_APPDATA                 = 0x001a, // Version 4.71 以降： 「Application Data」（ファイルシステムディレクトリ）
	CSIDL_PRINTHOOD               = 0x001b, // 「プリンタ」仮想フォルダにおかれるリンクオブジェクトを格納するフォルダ（ファイルシステムディレクトリ）
	CSIDL_LOCAL_APPDATA           = 0x001c, // Version 5.0 以降： 「Application Data」（ファイルシステムディレクトリ）
	CSIDL_ALTSTARTUP              = 0x001d, // 非ローカル版の「スタートアップ」（ファイルシステムディレクトリ）
	CSIDL_COMMON_ALTSTARTUP       = 0x001e, // 非ローカル版の AllUsers の「スタートアップ」（ファイルシステムディレクトリ）
	CSIDL_COMMON_FAVORITES        = 0x001f, // AllUsers の「お気に入り」（NT系のみ）（ファイルシステムディレクトリ）
	CSIDL_INTERNET_CACHE          = 0x0020, // Version 4.72 以降： インターネット一時ファイルを格納するフォルダ（ファイルシステムディレクトリ）
	CSIDL_COOKIES                 = 0x0021, // 「Cookies」（ファイルシステムディレクトリ）
	CSIDL_HISTORY                 = 0x0022, // 「履歴」（ファイルシステムディレクトリ）
	CSIDL_COMMON_APPDATA          = 0x0023, // Version 5.0 以降： AllUsers の「Application Data」（ファイルシステムディレクトリ）
	CSIDL_WINDOWS                 = 0x0024, // Version 5.0 以降： Windows ディレクトリ
	CSIDL_SYSTEM                  = 0x0025, // Version 5.0 以降： Windows System ディレクトリ
	CSIDL_PROGRAM_FILES           = 0x0026, // Version 5.0 以降： Program Files フォルダ
	CSIDL_MYPICTURES              = 0x0027, // Version 5.0 以降： My Pictures フォルダ（ファイルシステムディレクトリ）
	CSIDL_PROFILE                 = 0x0028, // Version 5.0 以降： profile フォルダ
	CSIDL_PROGRAM_FILES_COMMON    = 0x002b, // Version 5.0 以降；Windows 2000/XP： Program Files\Common
	CSIDL_COMMON_TEMPLATES        = 0x002d, // Windows NT/2000/XP： AllUsers のドキュメントテンプレートが格納されるディレクトリ
	CSIDL_COMMON_DOCUMENTS        = 0x002e, // Shfolder.dll： AllUsers のドキュメントテンプレートが格納されるディレクトリ（Windows NT 系および Shfolder.dll がインストールされた Windows 9x）
	CSIDL_COMMON_ADMINTOOLS       = 0x002f, // Version 5.0 以降： AllUsers の管理ツールディレクトリ
	CSIDL_ADMINTOOLS              = 0x0030, // Version 5.0 以降： 管理ツールディレクトリ
	CSIDL_COMMON_MUSIC    = 0x35,
	CSIDL_COMMON_PICTURES = 0x36,
	CSIDL_COMMON_VIDEO    = 0x37,
+/
	CSIDL_MYMUSIC                 = 0x000d, // 「マイ ミュージック」（ファイルシステムディレクトリ）
	CSIDL_MYVIDEO                 = 0x000e, // 
}
