﻿/**

悪いこと言わないからstd.getopt使うことをお勧めするよ。

History:
	$(HIST_VER 1.100,
		$(HIST_NEW)
	)
*/
module etc.negui.input.commandline;

debug import std.stdio: wl = writefln;
debug(commandline) void main() {}

import std.contracts;

import win32.core;

import etc.negui.system.base;
public import etc.negui.input.inputif;

///
class CommandLineException: InputException {
	mixin MixInNeGuiException;
}

/**
コマンドライン引数。


*/
class CommandLine {
	/// コマンドライン起動パス。
	const Text executePath;
	/// コマンドライン引数全て。
	const Text[] args;

	/// オプションの先頭。
	Text option={data:"--"w.dup};
	/// オプションの区切り。
	Text pairSplit={data:"="w.dup};

	/**
	Params:
		CommandLineSrc = C形式のコマンドライン文字列。

		ExecutePath = CommandLineSrcの先頭を実行パスとして扱うか。
	*/
	this(in wchar* CommandLineSrc, bool ExecutePath) {
		LONG argc;
		auto argv=CommandLineToArgvW(CommandLineSrc, &argc);
		scope(success) LocalFree(cast(HANDLE)argv);
		enforce(argv, new CommandLineException(ERR.toText));
		assert(argc);

		Text[] Args;
		size_t StartIndex;
		if(ExecutePath) {
			executePath = Text(argv[0]);
			Args.length = argc-1;
			StartIndex = 1;
		} else {
			executePath = Text.init;
			Args.length = argc;
			StartIndex = 0;
		}
		foreach(i, ref Arg; Args) {
			Arg = Text(argv[StartIndex + i]);
		}
		args = Args;
	}

	this() {
		this(GetCommandLine(), true);
	}
	
	this(in Text Value, bool ExecutePath=false) {
		this(Value.ptr, ExecutePath);
	}

	/***/
	static Text[][Text] pairs(in Text[] Args, in Text Option, in Text PairSplit) {
		Text[][Text] Pairs;
		
		foreach(ref const(Text) Arg; Args) {
			if(Arg.startsWith(Option) && Arg.length > Option.length) {
				const Singel=Arg[Option.length .. Arg.length];
				//auto SplitIndex=Singel.find(PairSplit);
				auto SplitIndex=Singel.indexOf(PairSplit, Text.CASE_SENSITIVE.YES, Text.START.HEAD);
				if(SplitIndex == -1 || !SplitIndex) {
					continue;
				}
				auto Key=Singel[0 .. SplitIndex];
				auto Value=Singel[SplitIndex+1 .. Singel.length];
				if(Value.length) {
					Pairs[Key] ~= Value;
				}
			}
		}
		return Pairs;
	}
	debug(commandline) unittest {
		auto p=CommandLine.pairs(Texts("--123=abc", "-123=abc", "--456=ABC", "--123=ABC", "--123=", "--=test"), Text("--"), Text("="));
		assert(p[Text("123")].length == 2);
	}
	/**
	オプションのペア形式を取得。
	*/
	const Text[][Text] pairs() {
		return pairs(args, option, pairSplit);
	}
	debug(commandline) unittest {
		auto cl=new CommandLine(Text("--123=abc -123=abc --456=ABC --123=ABC --123= --=test"));
		assert(cl.pairs[Text("123")].length == 2);
	}

	/***/
	static Text[] singles(in Text[] Args, in Text Option) {
		Text[] Singles;
		
		foreach(ref Text Arg; Args) {
			if(!Arg.startsWith(Option)) {
				Singles ~= Arg.dup;
			}
		}
		return Singles;
	}
	debug(commandline) unittest {
		auto p=CommandLine.singles(Texts("--123=abc", "-123=abc", "--456=ABC", "--123=ABC", "--123=", "--=test"), Text("--"));
		assert(p.length == 1);
		assert(p[0] == Text("-123=abc"));
	}
	/**
	オプション形式以外を取得。
	*/
	const Text[] singles() {
		return singles(args, option);
	}
	debug(commandline) unittest {
		auto cl=new CommandLine(Text("--123=abc -123=abc --456=ABC --123=ABC --123= --=test"));
		assert(cl.singles[0] == Text("-123=abc"));
	}

	static Text[] options(in Text[] Args, in Text Option, in Text PairSplit) {
		Text[] Options;
		
		foreach(ref Text Arg; Args) {
			if(Arg.startsWith(Option) && Arg.length > Option.length) {
				//auto SplitIndex=Arg.find(PairSplit);
				auto SplitIndex=Arg.indexOf(PairSplit, Text.CASE_SENSITIVE.YES, Text.START.HEAD);
				if(SplitIndex == -1) {
					Options ~= Arg[Option.length .. Arg.length];
				}
			}
		}
		return Options;
	}
	debug(commandline) unittest {
		auto p=CommandLine.options(Texts("--123=abc", "-123=abc", "--456=ABC", "--123=ABC", "--123=", "--=test"), Text("--"), Text("="));
		assert(p.length == 0);
	}
	/**
	オプション形式を取得。

	ペア形式は取得しない。
	*/
	const Text[] options() {
		return options(args, option, pairSplit);
	}
	debug(commandline) unittest {
		auto cl=new CommandLine(Text("--abc --123=abc -123=abc --456=ABC --123=ABC --123= --=test"));
		assert(cl.options[0] == Text("abc"));
	}

}


