/**
*/
module etc.negui.system._hide.text;

debug import std.stdio: wl = writefln;

import std.c.string;

import std.array;
import std.traits;
import std.contracts;
import std.uni;
import std.utf;
import std.windows.charset;

import etc.negui.system.text;
//import etc.negui.system.exception;

///
bool _D_ANSI(string s) {
	try {
		validate(s);
		return true;
	} catch(Object o) {
		return false;
	}
}
///
string toANSI(string s)
in {
	assert(_D_ANSI(s));
}
body {
	auto p=toMBSz(s);
	auto len=strlen(p);
	return cast(string)p[0 .. len];
}
///
string toD(string s)
out(r) {
	assert(_D_ANSI(r));
}
body {
	return fromMBSz(s.ptr);
}

private immutable {
	auto NUMERAL_ZERO_CHAR=
		"0123456789"
		"ABCDEFGHIJKLM"
		"NOPQRSTUVWXYZ"
	;
	auto NUMERAL_ZERO_NUM = [
		0, 1, 2, 3, 4, 5, 6, 7, 8, 9,
		11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23,
		24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36
	];
	static assert(NUMERAL_ZERO_CHAR.length == NUMERAL_ZERO_NUM.length);
}
/**
文字からn進数取得。

Params:
	c = n進数の最大値を表す文字。
	[0-9A-Za-z]。

Returns:
	n進数。

Throws:
	失敗時にNeGuiException。
*/
static int numeralChatToNum(char c) {
	int n;
	if('0' <= c && c <= '9') {
		n = c - '0';
	} else {
		n = std.string.indexOf(NUMERAL_ZERO_CHAR, c, std.string.CaseSensitive.no);
	}
	enforce(n >= 0, std.string.format("c -> %s in [%s]", c, NUMERAL_ZERO_CHAR));
	
	return NUMERAL_ZERO_NUM[n];
}





/**
きちんと試験してないから動くか分からん。
*/
private void T_ToCharConv(C, T)(ref C[] s, T fd)
if(isSomeChar!(C) && is(T == delegate) || is(T == return))
in {
	assert(fd);
}
body {
	static if(is(C == char)) {
		C[4] Buffer;
	} else static if(is(C == wchar)) {
		C[2] Buffer;
	}
	
	for(auto i=0; i < s.length; i++) {
		immutable c=fd(s[i]);
		static if(is(C == dchar)) {
			s[i] = c;
		} else {
			immutable Length=encode(Buffer, c);
			//assert(cast(tchar)c == c, "辻褄合わせ未実装");
			if(Length == 1) {
				s[i] = cast(C)c;
			} else {
				s[i .. i+1] = Buffer;
			}
		}
	}
}

void ToCharConv(C)(ref C[] s, dchar function(dchar) fn) {
	T_ToCharConv(s, fn);
}
void ToCharConv(C)(ref C[] s, dchar delegate(dchar) dg) {
	T_ToCharConv(s, dg);
}



/**
大文字小文字を区別する
*/
bool DistinguishMatch(dchar c1, dchar c2) {
	return c1 == c2;
}
/**
大文字小文字を区別しない
*/
bool IgnoreMatch(dchar c1, dchar c2) {
	if(isUniAlpha(c1) && isUniAlpha(c2)) {
		return toUniUpper(c1) == toUniUpper(c2);
	} else {
		return DistinguishMatch(c1, c2);
	}
}

/**
大文字・小文字の区別。
*/
enum _CASE_SENSITIVE {
	YES, /// 区別する。
	NO,  /// 区別しない。
}
/**
開始点。
*/
enum _START {
	HEAD, /// 先頭から。
	TAIL, /// 末尾がら。
}


///
pure nothrow bool function(dchar, dchar) GetCaceSensitiveFunc(_CASE_SENSITIVE CaseSensitive) {
	final switch(CaseSensitive) {
		case _CASE_SENSITIVE.YES: return &DistinguishMatch;
		case _CASE_SENSITIVE.NO:  return &IgnoreMatch;
	}
}


/**
History:
	$(HIST_VER 1.100,
		$(HIST_F 文字列に対応。)
		$(HIST_B std.string.lastIndexOfの挙動と違う。)
		$(HIST_S 引数変更。)
	)
*/
static int Find(C)(in C[] SrcWord, in C[] SrcSub, _CASE_SENSITIVE CaseSensitive, _START Start)
if(isSomeChar!(C))
body {
	if(!SrcSub.length || SrcSub.length != 1 && SrcWord.length < SrcSub.length) {
		return -1;
	}

	auto Match=hide_text.GetCaceSensitiveFunc(CaseSensitive);
	immutable(C)[] Word, Sub;
	final switch(Start) {
		case Text.START.HEAD:
			Word = SrcWord.idup;
			Sub  = SrcSub.idup;
			break;
		case Text.START.TAIL:
			Word = SrcWord.idup;
			Sub  = SrcSub.idup;
			Word.reverse;
			Sub.reverse;
			break;
	}

	immutable MaxCount=Sub.length;
	bool Hit=true;
	for(auto i=0, j=0; i < Word.length || !Hit && Word.length < MaxCount; i++) {
		Hit = Match(Word[i], Sub[j]);
		if(Hit) {
			if(++j == MaxCount) {
				if(Start == Text.START.HEAD) {
					return i - (j - 1);
				} else {
					return Word.length - i - 1;
				}
			}
		} else {
			j = 0;
		}
	}

	return -1;
}






enum FILTERING {
	FILTER,
	EXTRACT,
}
C[] Filtering(C)(in C[] Src, in C[] Characters, FILTERING filtering)
if(isSomeChar!(C))
in {
	assert(Characters.length);
}
body {
	size_t i, j;
	auto Result=new C[Src.length];

	auto Exp = filtering == FILTERING.FILTER
		? (C a, C b) { return a == b;}
		: (C a, C b) { return a != b;}
	;

	while(j < Result.length) {
		immutable Target=Src[j];

		bool Hit=false;
		foreach(c; Characters) {
			if(Exp(c, Target)) {
				Hit = true;
				break;
			}
		}

		if(Hit) {
			j++;
		} else {
			Result[i++] = Src[j++];
		}
	}

	return Result[0 .. i];
}

__EOF__;



