﻿/**
NeGuiの文字列。


*/
module etc.negui.system.text;

debug import std.stdio: wl = writefln;
debug(text) void main() {}

import std.array;
static import std.algorithm;
import std.math;
import std.conv;
import std.utf;
import std.format;
static import std.string;
import std.ctype;
import std.uni;
import std.traits;
import std.contracts;
import std.typetuple;

import etc.negui.builder;

import hide_text=etc.negui.system._hide.text;

import win32.winbase;


/**
文字及び文字列。
*/
version(Unicode) {
	alias wchar tchar;
} else {
	alias char tchar;
}
/// ditto
alias immutable(tchar)[] tstring;

/**
文字列。



History:
	$(HIST_VER 1.120,
		$(HIST_F 大幅書き直し, char, wchar, dcharに多分対応。)
		$(HIST_O ところどころHistoryがなくなったり更新されてないけど気にしない。)
	)
*/
struct _NTEXT(CHARACTER, C=Unqual!(CHARACTER), A=C[], S=immutable(C)[])
if(isSomeChar!(C))
{
	///
	A data=null;
	//alias data this;
	bool emptyIsNull=false;

	private alias typeof(this) _TEXT;
	
	/**
	型。
	*/
	static nothrow pure TypeInfo charType() {
		return typeid(C);
	}
	/// ditto
	static nothrow pure TypeInfo textType() {
		return typeid(A);
	}

	immutable _TEXT newline={ data: cast(A)"\r\n"};


	/**
	ラップ。

	tchar[]そのものをTextに取り込む。

	Patams:
		arg = 取り込む配列。

	Returns:
		ラップ後のText。

	Examples:
	* --------------------------------
	* tstring src="abc";
	*
	* tchar[] buffer=src.dup;
	* auto t=Text.wrap(buffer);
	* 
	* t[2] = 'C';
	* assert(buffer == "abC");
	* assert(buffer is t.text);
	* --------------------------------
	
	History:
		$(HIST_VER 1.100,
			$(HIST_NEW)
		)
	*/
	static ref _TEXT wrap(A arg) {
		auto t=new _TEXT;
		t.data = arg;
		return *t;
	}
	debug(text) unittest {
		S src="abc";
		
		A buffer=src.dup;
		auto t=_TEXT.wrap(buffer);
		
		t[2] = 'C';
		assert(buffer == "abC");
		assert(buffer is t.data);
	}
	/**
	ラップ。

	Text.wrapの書き換えないバージョン。
	
	History:
		$(HIST_VER 1.110,
			$(HIST_CHG 属性, static pure ref, static pure)
		)
		$(HIST_VER 1.100,
			$(HIST_NEW)
		)
	*/
	static pure const(_TEXT) iwrap(S arg) {
		/+
		auto t=new Text;
		t.text = cast(wchar[])arg;
		return *t;
		+/
		_TEXT t;
		t.data=cast(A)arg;
		return t;
	}
	debug(text) unittest {
		auto t=_TEXT.iwrap("abcd");
		// t[0] = 'A'; // <- Error
		assert(t[0] == 'a');
		assert(t.length == 4);
	}

	
	/**
	History:
		$(HIST_VER 1.120,
			$(HIST_NEW)
		)
	*/
	const UtfString toUTF(T=char, UtfString=immutable(T)[])()
	if(isSomeChar!(T))
	{
		return to!(UtfString)(data);
	}
	debug(text) unittest {
		auto t=_TEXT("aaa");
		assert(t.toUTF!(char)=="aaa"c);
		assert(t.toUTF!(wchar)=="aaa"w);
		assert(t.toUTF!(dchar)=="aaa"d);
	}

	/**
	C形式文字列へ変換。

	Returns:
		終端0文字列。
		それっぽいのはText.ptrを使用すべき。

	History:
		$(HIST_VER 1.120,
			$(HIST_P toUTFの利用。)
		)
		$(HIST_VER 1.110,
			$(HIST_P テンプレートへ変更。)
		)
	*/
	const const(T)* toStringz(T=char)()
	if(isSomeChar!(T))
	{
		return (toUTF!(T) ~ '\0').ptr;
	}
	debug(text) unittest {
		_TEXT t;
		t = _TEXT("abc");
		assert(t == _TEXT(t.toStringz!(char)));
		assert(t == _TEXT(t.toStringz!(wchar)));
		assert(t == _TEXT(t.toStringz!(dchar)));
	}

	private const string _toString() {
		return toUTF!(char);
	}
	const string str() {
		return _toString;
	}
	const string toString() {
		return _toString;
	}
	string toString() {
		return _toString;
	}

	/**
	*/
	const string toANSI() {
		return hide_text.toANSI(_toString);
	}
	/**
	*/
	const char* toANSIz() {
		return cast(char*)std.string.toStringz(hide_text.toANSI(_toString));
	}
	/**
	*/
	static ref _TEXT convertText(string AnsiString) {
		return _TEXT(hide_text.toD(AnsiString));
	}

	

	/**
	Windows用文字列の生成。

	Params:
		EmptyIsNull = trueの場合に文字列が空ならnullを返す。

	Returns:
		dataのC形式(0終端)文字列を生成。
		誰かに渡さないとGCの対象になるんで注意。

	History:
		$(HIST_VER 1.100,
			$(HIST_P 挙動の変更。)
			$(HIST_S 引数追加。)
		)
	*/
	const C* ptr(bool EmptyIsNull) {
		if(EmptyIsNull && !data.length) {
			return null;
		}
		static if(is(C == char)) {
			return toANSIz;
		} else static if(is(C == wchar)) {
			return (data.dup ~ cast(wchar)0).ptr;
		} else static if(is(C == dchar)) {
			assert(false, "windows api -> char or wchar");
		}
	}
	/**
	Windows用文字列の生成。

	Returns:
		emptyIsNullに影響される。
	*/
	const C* ptr() {
		return ptr(emptyIsNull);
	}
	debug(text) unittest {
		const Text t=Text("aaa");
		t.ptr;
	}
	const ref _TEXT dup() {
		auto t=new _TEXT;
		t.data = data.dup;
		return *t;
	}


	/**
	文字列比較。
	
	大文字小文字を区別しない。

	Params:
		text = 比較する文字列。
	
	History:
		$(HIST_VER 1.100,
			$(HIST_CHG 名前, cmpi, icmp)
		)
		$(HIST_VER 1.00β11,
			$(HIST_NEW)
		)
	*/
	const int icmp(in _TEXT text) {
		static if(is(C == char)) {
			return std.string.icmp(data, text.data);
		} else static if(is(C == wchar)) {
			return lstrcmpiW(toStringz!(wchar), text.toStringz!(wchar));
		} else static if(is(C == dchar)) {
			assert(false);
		}
	}

	
	///
	const hash_t toHash() {
		hash_t Hash;

		foreach(c; data) {
			Hash = (Hash * data.length) + c;
		}

		return Hash;
	}
	///
	const bool opEquals(const ref _TEXT arg) {
		return data == arg.data;
	}
	///
	const int opCmp(const ref _TEXT arg) {
		return std.string.cmp(this.data, arg.data);
	}

	///
	private static A TextFormat(...) {
		auto app=appender!(A);
		C[4 / C.sizeof] Buffer;
		void putc(dchar c) {
			static if(is(C == dchar)) {
				app.put(c);
			} else {
				immutable Length=encode(Buffer, c);
				app.put!(A)(Buffer[0 .. Length]);
			}
		}
		doFormat(&putc, _arguments, _argptr);
		return app.data;
	}
	debug(text) unittest {
		assert(TextFormat(cast(S)"%0.*s", 10, 59) == "0000000059");
	}
	/**
	文字からText生成。
	*/
	static ref _TEXT opCall(T: dchar)(in T arg) {
		auto ResultText=new _TEXT;
		static if(is(C == dchar)) {
			ResultText.data = [arg];
		} else {
			encode(ResultText.data, arg);
		}
		return *ResultText;
	}
	debug(text) unittest {
		_TEXT t;
		t = _TEXT('a');
		assert(t.data == "a");
		t = _TEXT('あ');
		assert(t.data == "あ");
	}
	/**
	C形式文字列からText生成。
	*/
	static ref _TEXT opCall(T)(in T* arg)
	if(isSomeChar!(T))
	{
		auto ResultText=new _TEXT;
		
		if(arg !is null) {
			size_t i=0;
			while(arg[i]) i++;
			
			ResultText.data= to!(S)(arg[0 .. i]).dup;
		} else {
			ResultText.data = null;
		}
		
		return *ResultText;
	}
	debug(text) unittest {
		_TEXT t;
		
		t = _TEXT(x"313233003435360037383900"c.ptr);
		assert(t.data == "123");
		t = _TEXT(x"313233003435360037383900"w.ptr);
		assert(t.data == "123");
		t = _TEXT(x"313233003435360037383900"d.ptr);
		assert(t.data == "123");
	}
	/**
	クラスからText生成。
	
	Params:
		arg = 変換するクラス。
		      ITextを継承していること。
	*/
	static ref _TEXT opCall(T)(in T arg)
	if(is(T: IText))
	{
		if(arg) {
			return arg.toText.dup;
		} else {
			return Text("null");
		}
	}
	debug(text) unittest {
		class itext: IText {
			const Text toText() {
				return Text("aaa");
			}
			override string toString() {
				return "bbb";
			}
		}
		auto o=new itext();
		auto t=Text(o);
		t == Text("aaa");
	}
	/**
	構造体からText生成。

	Params:
		arg = 変換するクラス。
		      toTextを持っていること。
	*/
	static ref Text opCall(T)(const ref T arg)
	if(is(T == struct) && is(typeof(&T.init.toText)))
	{
		return arg.toText;
	}
	debug(text) unittest {
		struct STRUCT {
			string s;
			const ref Text toText() {
				return Text(s);
			}
		}
		STRUCT s;
		s.s="abc";
		auto t=_TEXT(s);
		assert(t.data, "abc");
	}

	/**
	TextからText生成。

	ただのコピー。

	Params:
		arg = 変換するText。
		      arg.dup()を内部で呼び出し。
	*/
	static ref _TEXT opCall(T: _TEXT)(const ref T arg) {
		return arg.dup;
	}
	debug(text) unittest {
		_TEXT t;
		t.data=(cast(S)"abc").dup;
		auto t2 = t.dup;
		assert(t.data == t2.data);
		assert(t.data !is t2.data);
	}
	static ref _TEXT opCall(T)(in T arg)
	if(isSomeString!(Unqual!(T)))
	{
		auto t=new _TEXT;
		t.data = (to!(S)(arg)).dup;
		return *t;
	}
	debug(text) unittest {
		_TEXT("abc");
	}
	/**
	引数からText生成。

	Params:
		arg = 何か。

	Note:
		引数属性がinなのでObject.toStringはできない。

		クラスを変換する場合はITextを継承したクラスにしてText!(IText)を使用するか、
		Text(String, T...)を使用する。
	*/
	static ref _TEXT opCall(T)(in T arg) {
		auto ResultText=new _TEXT;
		ResultText.data = (to!(S)(arg)).dup;
		return *ResultText;
	}
	debug(text) unittest {
		_TEXT t;
		t = _TEXT(`a`);
		assert(t.data == "a");
		t = _TEXT(123);
		assert(t.data == "123");

		_TEXT c = "c";
		_TEXT w = "w"w;
		_TEXT d = "d"d;
	}

	/**
	書式からText作成。
	*/
	static ref _TEXT opCall(String, T...)(in String SrcFormat, T Args) {
		auto ResultText=new _TEXT;

		static if(IsNTEXT!(String)) {
			auto FormatString = SrcFormat.data;
		} else static if(isSomeString!(String))  {
			auto FormatString = to!(S)(SrcFormat);
		} else {
			static assert("String is " ~ String.stringof);
		}
		ResultText.data = TextFormat(FormatString, Args);

		return *ResultText;
	}
	debug(text) unittest {
		_TEXT t;
		t=_TEXT("aaa");
		assert(t.data == "aaa");
		t=_TEXT("bbb"w);
		assert(t.data == "bbb");
		t=_TEXT("ccc"d);
		assert(t.data == "ccc");
		t=_TEXT(__VENDOR__.dup);
		assert(t == _TEXT(__VENDOR__));

		t=_TEXT("aaa\0"w.dup.ptr);
		assert(t.data == "aaa");
		t=_TEXT(_TEXT("vvv"));
		assert(t.data == "vvv");

		t=_TEXT(100);
		assert(t.data == "100");
		t=_TEXT(1.5);
		assert(t.data == "1.5");
		t=_TEXT(1.5);
		assert(t.data == "1.5");

		t=_TEXT("%s", __VENDOR__);
		assert(t == _TEXT(__VENDOR__));
		t=_TEXT("<%s>", __TIMESTAMP__);
		assert(t == _TEXT("<" ~ __TIMESTAMP__ ~ ">"));
		t=_TEXT(_TEXT("a=%05s, b=%s, c=%s"), 1, 1.5, "ABC");
		assert(t.data == "a=00001, b=1.5, c=ABC");
		t = _TEXT("<%s>", 100);
		assert(t.data == "<100>");
		t = _TEXT("<%s>"w, 100);
		assert(t.data == "<100>");
		t = _TEXT("<%s>"d, 100);
		assert(t.data == "<100>");
		t = _TEXT(_TEXT("<%s>"), 100);
		assert(t.data == "<100>");

		//wl(__FILE__);
		//? t = Text("%.*s",2,12);
		//assert(t.text == "<100>");
	}
	/// 連結。
	const ref _TEXT opCat(in _TEXT Value) {
		return _TEXT(data ~ Value.data);
	}
	debug(text) unittest {
		_TEXT t;
		t = _TEXT("123");
		assert((t ~ _TEXT("456")).data == "123456");
	}

	/// 自身へ連結。
	ref _TEXT opCatAssign(in _TEXT Value) {
		data ~= Value.data;
		return this;
	}
	///
	@property const pure nothrow size_t length() {
		return data.length;
	}
	///
	@property nothrow size_t length(size_t Length) {
		return data.length = Length;
	}
	
	/// [n]
	const C opIndex(size_t Index) {
		return data[Index];
	}
	/// [n] = value
	C opIndexAssign(C Character, size_t Index) {
		return data[Index] = Character;
	}

	///
	const A opIndex(size_t[] Index ...) {
		auto Characters=new C[Index.length];
		for(auto i=0; i < Index.length; i++) {
			Characters[i] = data[Index[i]];
		}
		return Characters;
	}
	///
	void opIndexAssign(C Character, size_t[] Index ...) {
		for(auto i=0; i < Index.length; i++) {
			data[Index[i]] = Character;
		}
	}

	/// []
	ref _TEXT opSlice() {
		return this;//Text(text[]);
	}
	/**
	[start .. end]

	History:
		$(HIST_VER 1.00β17,
			$(HIST_B 事前条件がDの仕様と違っていたのを修正。)
		)
	*/
	const ref _TEXT opSlice(in size_t start, in size_t end)
	in {
		assert(start <= end, std.string.format("%s[%s..%s]", data, start, end));
	}
	body {
		auto t=new _TEXT;
		t.data = data[start..end].dup;
		return *t;
	}
	debug(text) unittest {
		_TEXT t;
		t = _TEXT("LICENSE");
		assert(t[0..1].data=="L");
		assert(t[0..2].data=="LI");
		assert(_TEXT(t.data[0..3]).data == "LIC");
		assert(_TEXT(t.data[0..7]).data == "LICENSE");
	}


	/**
	*/
	const dchar characterGet(size_t Position) {
		static if(is(C == dchar)) {
			return data[Position];
		} else {
			auto Index=toUTFindex(data, Position);
			auto Length=stride(data, Index);
			if(Length == 1) {
				return data[Index];
			} else {
				size_t Next;
				return decode(data[Index .. Index + Length], Next);
			}
		}
	}
	debug(text) unittest {
		auto t=_TEXT("abcあいう");
		assert('a' == t.characterGet(0));
		assert('b' == t.characterGet(1));
		assert('c' == t.characterGet(2));
		assert('あ' == t.characterGet(3));
		assert('い' == t.characterGet(4));
		assert('う' == t.characterGet(5));
	}
	/**
	*/
	dchar characterSet(size_t Position, dchar Character) {
		dchar Old;
		static if(is(C == dchar)) {
			Old = data[Position];
			data[Position] = Character;
		} else {
			auto Index=toUTFindex(data, Position);
			auto Length=stride(data, Index);
			size_t Next;
			if(Length == 1) {
				Old = data[Index];
				Next = 1;
			} else {
				Old = decode(data[Index .. Index + Length], Next);
			}
			C[4 / C.sizeof] Buffer;
			auto BufferSize=encode(Buffer, Character);
			if(Position) {
				auto Count=toUTFindex(data, Position);
				data = data[0 .. Count] ~ Buffer[0 .. BufferSize] ~ data[Index+Next .. $];
			} else {
				data = Buffer[0 .. BufferSize] ~ data[Next .. $];
			}
		}
		return Old;
	}
	debug(text) unittest {
		auto t=_TEXT("abcあいう");
		assert('a' == t.characterSet(0, 'A'));
		assert(t == _TEXT("Abcあいう"));
		assert('b' == t.characterSet(1, 'B'));
		assert(t == _TEXT("ABcあいう"));
		assert('c' == t.characterSet(2, 'C'));
		assert(t == _TEXT("ABCあいう"));
		assert('あ' == t.characterSet(3, 'ア'));
		assert(t == _TEXT("ABCアいう"));
		assert('い' == t.characterSet(4, 'イ'));
		assert(t == _TEXT("ABCアイう"));
		assert('う' == t.characterSet(5, 'ウ'));
		assert(t == _TEXT("ABCアイウ"));
	}

	static _TEXT[] toArray(T...)(in T args) {
		if(!args.length) {
			return null;
		}

		auto ts=new _TEXT[args.length];

		foreach(i, arg; args) {
			ts[i] = _TEXT(arg);
		}

		return ts;
	}
	debug(text) unittest {
		auto t=_TEXT.toArray(1, 2, "3", '4', 5.0, "", "789");
		assert(t[0].data == "1");
		assert(t[1].data == "2");
		assert(t[2].data == "3");
		assert(t[3].data == "4");
		assert(t[4].data == "5");
		assert(t[5].data == "");
	}
	/// ditto
	static _TEXT[] toArray(T)(in T[] args) {
		if(!args.length) {
			return null;
		}

		auto ts=new _TEXT[args.length];

		for(auto i=0; i < ts.length; i++) {
			ts[i] = _TEXT(args[i]);
		}

		return ts;
	}


	
	/**
	Unicode文字列判定。

	現在の文字列がUnicodeで合法かどうかを判定。

	Returns:
		Unicodeとして正しければtrue、不正ならfalse。
	
	History:
		$(HIST_VER 1.100,
			$(HIST_NEW)
		)
	*/
	const bool validate() {
		return collectException(std.utf.validate(data)) is null;
	}
	/**
	文字列長取得。

	現在のUnicode文字列長を取得。
	length()と違い文字列の長さを取得する。

	Returns:
		文字列長。

	In:
		文字列がUnicodeとして正しいこと。
	*/
	version(none)@property const size_t characterLength()
	in {
		assert(validate());
	}
	body {
		return std.utf.count(data);
	}


	/**
	数値変換。

	現在の文字列からTに対応する数値を取得。

	Returns:
		Tに対応する数値。

	Throws:
		変換に失敗した場合はException。

	Examples:
	* --------------------------------
	* auto t=Text("123");
	* assert(t.toNumber!(int) == 123);
	* --------------------------------
	
	History:
		$(HIST_VER 1.100,
			$(HIST_NEW)
		)
	*/
	const T toNumber(T, string _file=__FILE__, int _line=__LINE__)()
	if(std.traits.isNumeric!(T))
	{
		try {
			return to!(T)(data);
		} catch(Throwable e) {
			throw new Exception(std.string.format("%s toNumber(), data -> %s", T.stringof, data), _file, _line, e);
		}
	}
	debug(text) unittest {
		_TEXT t;
		t = _TEXT("123");
		assert(t.toNumber!(int) == 123);
		t = _TEXT("123");
		assert(t.toNumber!(byte) == 123);
		t = _TEXT("123");
		assert(t.toNumber!(byte) == 123);
		t = _TEXT("1.1");
		assert(t.toNumber!(real) == 1.1);
	}
	/**
	数値変換。

	toNumberと違い例外出力を行わない。

	Params:
		Value = 変換後の数値。

	Returns:
		変換成功の真偽値。
	
	Examples:
	* --------------------------------
	* auto t=_TEXT("123");
	* int result;
	* if(t.tryNumber!(int)(result)) {
	* 	assert(result == 123);
	* }
	* --------------------------------
	
	History:
		$(HIST_VER 1.100,
			$(HIST_NEW)
		)
	*/
	const bool tryNumber(T)(out T Value) {
		try {
			Value = toNumber!(T);
			return true;
		} catch {
			return false;
		}
	}
	debug(text) unittest {
		_TEXT t;
		t = _TEXT("123");
		int a;
		assert(t.tryNumber!(int)(a));
		assert(!t.init.tryNumber!(int)(a));
	}
	
	/**
	数値に区切り追加。

	1000を1,000のように変換。

	Params:
		number = 変換する数値。

		SplitChar = 区切りにする文字。

		SplitCount = SplitCharを設定する桁数。

	Returns:
		区切り文字の追加された文字列。

	In:
		SplitCount > 0
	
	Examples:
	* --------------------------------
	* assert(Text.numberSplit(1000) == Text("1,000"));
	* assert(Text.numberSplit(1000, '.') == Text("1.000"));
	* assert(Text.numberSplit(1000, ',', 1) == Text("1,0,0,0"));
	* --------------------------------
	
	History:
		$(HIST_VER 1.100,
			$(HIST_NEW)
		)
	*/
	static ref _TEXT numberSplit(T)(T number, C SplitChar=',', size_t SplitCount=3)
	if(isIntegral!(T))
	in {
		assert(SplitCount > 0);
	}
	body {
		auto s=to!(string)(number);
		immutable Minus=(number < 0) ? 1: 0;
		
		if(s.length - Minus <= SplitCount) {
			return _TEXT(s);
		}
		
		immutable Counter=(s.reverse.length - 1 - Minus) / SplitCount;
		auto t=new C[s.length + Counter];

		for(auto i=0, j=0; i < s.length; i++, j++) {
			if(i && !(i % SplitCount)) {
				t[j++] = SplitChar;
			}
			t[j] = s[i];
		}
		
		return wrap(t.reverse);
	}
	debug(text) unittest {
		assert(_TEXT.numberSplit(1000) == _TEXT("1,000"));
		assert(_TEXT.numberSplit(1000, '.') == _TEXT("1.000"));
		assert(_TEXT.numberSplit(1000, ',', 1) == _TEXT("1,0,0,0"));
	
		assert(numberSplit(1) == _TEXT("1"));
		assert(numberSplit(0) == _TEXT("0"));
		assert(numberSplit(100) == _TEXT("100"));
		assert(numberSplit(-100) == _TEXT("-100"), numberSplit(-100).toString);
		assert(numberSplit(999) == _TEXT("999"));
		assert(numberSplit(1000) == _TEXT("1,000"));
		assert(numberSplit(-1000) == _TEXT("-1,000"), numberSplit(-1000).toString);
		assert(numberSplit(10000) == _TEXT("10,000"));
		assert(numberSplit(100000) == _TEXT("100,000"));
		assert(numberSplit(1000000) == _TEXT("1,000,000"));
		assert(numberSplit(10000000) == _TEXT("10,000,000"));
		assert(numberSplit(100000000) == _TEXT("100,000,000"));
		assert(numberSplit(1000000000) == _TEXT("1,000,000,000"));
		assert(numberSplit(-1000000000) == _TEXT("-1,000,000,000"));
		
		assert(numberSplit(1000000000, '.') == _TEXT("1.000.000.000"));
		assert(numberSplit(1000000000, '.', 1) == _TEXT("1.0.0.0.0.0.0.0.0.0"));
		assert(numberSplit(1000000000, ':', 10) == _TEXT("1000000000"));
		assert(numberSplit(10000000000, ':', 10) == _TEXT("1:0000000000"));
		assert(numberSplit(-10000000000, ':', 10) == _TEXT("-1:0000000000"));
	}

	/**
	数値をn進数に変換。

	Params:
		number = 変換する数値。
	
		n = 変換するn進数。

		c = n進数の最大値を表す文字。

	Returns:
		変換した文字列。
	
	In:
		n <= 2 とか。

	History:
		$(HIST_VER 1.110,
			$(HIST_S テンプレートちまちま。)
		)
		$(HIST_VER 1.100,
			$(HIST_NEW)
		)
	*/
	static ref _TEXT toNumeral(T: int, BASE=int)(in T number, in BASE n=16)
	if(!isSomeChar!(BASE))
	in {
		assert(2 <= n && n <= hide_text.NUMERAL_ZERO_CHAR.length);
	}
	body {
		Unqual!(T) Number=number;
		auto app=appender!(A);
		size_t i;
		for(; Number; i++, Number /= n) {
			//t.autoIncrement(i, Increment)[i] = hide_text.NUMERAL_ZERO_CHAR[Number % n];
			app.put(hide_text.NUMERAL_ZERO_CHAR[Number % n]);
		}
		return wrap(app.data[0..i].reverse);
	}
	/// ditto
	static ref _TEXT toNumeral(T: int, BASE=tchar)(in T number, in BASE c='F')
	if(isSomeChar!(BASE))
	{
		return toNumeral(number, hide_text.numeralChatToNum(c));
	}
	/**
	n進数から10進数へ変換。
	
	Params:
		number = 変換する数値の文字列。
	
		n = n進数。

		c = n進数の最大値を表す文字。
	
	History:
		$(HIST_VER 1.110,
			$(HIST_S テンプレートちまちま。)
		)
	*/
	static T toNumeral10(T: int, BASE=int)(in _TEXT Numeral, BASE n)
	if(!isSomeChar!(BASE))
	in {
		assert(2 <= n && n <= hide_text.NUMERAL_ZERO_CHAR.length);
	}
	body {
		T Number;
		size_t i = Numeral.data.length - 1;
		foreach(c; Numeral.data) {
			Number += hide_text.numeralChatToNum(cast(char)c) * pow(cast(int)n, i--);
		}
		return Number;
	}
	/// ditto
	static T toNumeral10(T: int, BASE=tchar)(in _TEXT Numeral, BASE c)
	if(isSomeChar!(BASE))
	{
		return toNumeral10!(T)(Numeral, hide_text.numeralChatToNum(c));
	}
	debug(text) unittest {
		assert(toNumeral(2, 2).data == "10");
		assert(toNumeral(3, 2).data == "11");
		assert(toNumeral(4, 2).data == "100");
		assert(toNumeral(4, 16).data == "4");
		assert(toNumeral(10, 16).data == "A");
		assert(toNumeral(15, 16).data == "F");
		assert(toNumeral(16, 16).data == "10");
		assert(toNumeral(35, 36).data == "Z");
		assert(toNumeral(36, 36).data == "10");
		assert(toNumeral(2, '2').data == "10");
		assert(toNumeral(3, '2').data == "11");
		assert(toNumeral(4, '2').data == "100");
		assert(toNumeral(4, 'f').data == "4");
		assert(toNumeral(10, 'F').data == "A");
		assert(toNumeral(15, 'f').data == "F",toNumeral(15, 'f').toString);
		assert(toNumeral(16, 'F').data == "10");
		assert(toNumeral(35, 'Z').data == "Z");
		assert(toNumeral(36, 'z').data == "10");
		assert(toNumeral10!(int)(_TEXT("10"), 16) == 16);
		assert(toNumeral10!(int)(toNumeral(16, 16), 16) == 16);
		assert(toNumeral10!(int)(toNumeral(36, 'Z'), 36) == 36);
		assert(toNumeral10!(int,char)(toNumeral(36, 'Z'), 'z') == 36);
	}

	/**
	文字列からDelimiter削除。

	Params:
		Delimiter = 取り去る文字列。
		            Text.initならCR,LF,CRLF、

	Returns:
		処理後の文字列。

	Note:
		まんまstd.string.chomp。
	
	History:
		$(HIST_VER 1.100,
			$(HIST_S 引数変更)
		)
	*/
	const ref _TEXT chomp(in _TEXT Delimiter=_TEXT.init) {
		if(Delimiter.length) {
			return _TEXT(std.string.chomp(data, Delimiter.data));
		} else {
			return _TEXT(std.string.chomp(data));
		}
	}
	debug(text) unittest {
		_TEXT t="abc\r\n";
		_TEXT tc;

		tc=t.chomp;
		assert(tc == _TEXT("abc"));

		t = _TEXT("abcd");
		tc=t.chomp(_TEXT("d"));
		assert(tc == _TEXT("abc"));
	}

	/**
	先頭空白文字削除。

	Returns:
		処理後の文字列。
	*/
	const ref _TEXT stripl() {
		return _TEXT(std.string.stripl(data));
	}
	debug(text) unittest {
		_TEXT t=" abc";
		assert(t.stripl.data == "abc", t.toString());
		t = _TEXT("                              abc");
		assert(t.stripl.data == "abc", t.toString());
		t = _TEXT("\t\t\t\t\tabc");
		assert(t.stripl.data == "abc", t.toString());
		t = _TEXT("\r\r\r\r\rabc");
		assert(t.stripl.data == "abc", t.toString());
		t = _TEXT("\n\n\n\n\nabc");
		assert(t.stripl.data == "abc", t.toString());
		t = _TEXT(" ");
		assert(t.stripl.data == "", t.toString());
	}
	/**
	末尾空白文字削除。
	
	Returns:
		処理後の文字列。
	*/
	const ref _TEXT stripr() {
		return _TEXT(std.string.stripr(data));
	}
	debug(text) unittest {
		_TEXT t="abc ";
		assert(t.stripr.data == "abc", t.toString());
		t = _TEXT("abc                              ");
		assert(t.stripr.data == "abc", t.toString());
		t = _TEXT("abc\t\t\t\t\t");
		assert(t.stripr.data == "abc", t.toString());
		t = _TEXT("abc\r\r\r\r\r");
		assert(t.stripr.data == "abc", t.toString());
		t = _TEXT("abc\n\n\n\n\n");
		assert(t.stripr.data == "abc", t.toString());
		t = _TEXT(" ");
		assert(t.stripr.data == "", t.toString());
	}
	/**
	両端空白文字削除。
	
	Returns:
		処理後の文字列。
	
	History:
		$(HIST_VER 1.010,
			$(HIST_S 属性変更。)
		)
	*/
	const ref _TEXT strip() {
		return _TEXT(std.string.strip(data));
	}
	debug(text) unittest {
		_TEXT t="      abc      ";
		assert(t.strip.data == "abc", t.toString());
	}

	/**
	大文字・小文字の区別。
	*/
	alias hide_text._CASE_SENSITIVE CASE_SENSITIVE;
	/**
	開始点。
	*/
	alias hide_text._START START;

	/**
	*/
	_TEXT toUpperRaw() {
		hide_text.ToCharConv(data, &toUniUpper);
		return this;
	}
	debug(text) unittest {
		_TEXT t;
		t = _TEXT("a");
		assert(t.toUpperRaw == _TEXT("A"));
		t = _TEXT("abc");
		assert(t.toUpperRaw == _TEXT("ABC"));
		t = _TEXT("aBc");
		assert(t.toUpperRaw == _TEXT("ABC"));
		t = _TEXT("");
		assert(t.toUpperRaw == _TEXT(""));
	}
	/**
	*/
	_TEXT toLowerRaw() {
		hide_text.ToCharConv(data, &toUniLower);
		return this;
	}
	debug(text) unittest {
		_TEXT t;
		t = _TEXT("a");
		assert(t.toLowerRaw == _TEXT("a"));
		t = _TEXT("abc");
		assert(t.toLowerRaw == _TEXT("abc"));
		t = _TEXT("aBc");
		assert(t.toLowerRaw == _TEXT("abc"));
		t = _TEXT("");
		assert(t.toLowerRaw == _TEXT(""));
	}
	
	/**
	*/
	const _TEXT toUpper() {
		return dup.toUpperRaw();
	}
	/**
	*/
	const _TEXT toLower() {
		return dup.toLowerRaw();
	}

	/**
	*/
	const size_t count(C c) {
		size_t Counter=0;

		foreach(t; data) {
			if(t == c) {
				Counter++;
			}
		}
		
		return Counter;
	}
	debug(text) unittest {
		_TEXT t;
		
		t = _TEXT("abc;d;e;;f;g");
		assert(t.count(';') == 5);

		t = _TEXT("abcdefg");
		assert(t.count(';') == 0);

		t=_TEXT("");
		assert(t.count(';') == 0);
		
		t=_TEXT(";");
		assert(t.count(';') == 1);
	}
	/**
	*/
	const size_t count(in _TEXT Word)
	in {
		assert(Word.length);
	}
	body {
		if(!data.length) {
			return 0;
		}
		
		immutable WordLength = Word.length;
		
		if(WordLength == 1) {
			return count(Word[0]);
		}

		size_t Count=0;
		for(auto i=0; i < data.length - WordLength+1; i++) {
			if(data[i .. i + WordLength] == Word.data) {
				i += WordLength-1;
				Count++;
				continue;
			}
		}

		return Count;
	}
	debug(text) unittest {
		_TEXT t;

		t=_TEXT("123aa456aa789");
		assert(t.count(_TEXT("aa")) == 2);
		
		t=_TEXT("123aa456aa789aa");
		assert(t.count(_TEXT("aa")) == 3);
		
		t=_TEXT("aaaaaaaa");
		assert(t.count(_TEXT("aa")) == 4);
		
		t=_TEXT("aaaaaaaaa");
		assert(t.count(_TEXT("aa")) == 4);
		
		t=_TEXT("");
		assert(t.count(_TEXT("aa")) == 0);
		
		t=_TEXT("あああ");
		assert(t.count(_TEXT("あ")) == 3);
	}


	/**
	埋めもの。
	
	History:
		$(HIST_VER 1.110,
			$(HIST_NEW)
		)
	*/
	enum FILL {
		LEFT  , /// 左。
		CENTER, /// 中心。
		RIGHT , /// 右。
	}
	/**
	文字で埋める。
	
	Textそのものに作用。

	Params:
		Fill = 元の文字列の配置。

		c = 埋める文字。

		Width = 最終的なサイズ。
		        FillがFILL.CENTERでWidthが奇数の場合、元の文字列は左寄り。

	Returns:
		埋められた文字列。
		Widthが文字列より少ない場合は元の文字列。
	
	History:
		$(HIST_VER 1.110,
			$(HIST_NEW)
		)
	*/
	ref _TEXT fillRaw(FILL Fill, C c, size_t Width) {
		if(data.length >= Width) {
			return this;
		}
		auto SpaceLength=Width - data.length;
		assert(SpaceLength);
		auto Buffer=new C[SpaceLength];
		Buffer[] = c;
		final switch(Fill) {
			case FILL.LEFT:
				data = Buffer ~ data;
				break;
			case FILL.RIGHT:
				data ~= Buffer;
				break;
			case FILL.CENTER:
				size_t center=Buffer.length / 2;
				size_t padding=0;
				if(center % 2) {
					padding=1;
				}
				data = Buffer[0 .. center] ~ data ~ Buffer[center .. $-padding];
				break;
		}
		return this;
	}
	debug(text) unittest {
		_TEXT t;
		t = _TEXT("abc");
		t.fillRaw(FILL.LEFT, ' ', 8);
		assert(t == _TEXT("     abc"));
		t = _TEXT("abc");
		t.fillRaw(FILL.RIGHT, ' ', 8);
		assert(t == _TEXT("abc     "));
		t = _TEXT("abc");
		t.fillRaw(FILL.CENTER, ' ', 8);
		assert(t == _TEXT("  abc   "));
	}
	/**
	文字で埋める。
	
	Params:
		Fill = 元の文字列の配置。

		c = 埋める文字。

		Width = 最終的なサイズ。
		        FillがFILL.CENTERでWidthが奇数の場合、元の文字列は左寄り。

	Returns:
		埋められた文字列。
		Widthが文字列より少ない場合は元の文字列。

	
	History:
		$(HIST_VER 1.120,
			$(HIST_B Stack Overflow。)
		)
		$(HIST_VER 1.110,
			$(HIST_NEW)
		)
	*/
	const ref _TEXT fill(FILL Fill, C c, size_t Width) {
		auto t=dup;
		return t.fillRaw(Fill, c, Width);
	}


	/**
	文字検索。

	Params:
		c = 検索する文字。
	
		CaseSensitive = 大文字小文字の区別。
	
		Start = 検索位置。

	Returns:
		見つかった場合はそのインデックス、
		見つからなかった場合は-1を返す。

	Examples:
	* --------------------------------
	* Text t;
	* t = Text("123456789abcdABCD");
	*
	* assert(t.indexOf('1', CASE_SENSITIVE.YES, START.HEAD) == 0);
	* assert(t.indexOf('9', CASE_SENSITIVE.YES, START.HEAD) == 8);
	* assert(t.indexOf('a', CASE_SENSITIVE.YES, START.HEAD) == 9);
	* assert(t.indexOf('A', CASE_SENSITIVE.YES, START.HEAD) == 13);
	*
	* assert(t.indexOf('1', CASE_SENSITIVE.NO, START.HEAD) == 0);
	* assert(t.indexOf('9', CASE_SENSITIVE.NO, START.HEAD) == 8);
	* assert(t.indexOf('a', CASE_SENSITIVE.NO, START.HEAD) == 9);
	* assert(t.indexOf('A', CASE_SENSITIVE.NO, START.HEAD) == 9);
	*
	* assert(t.indexOf('D', CASE_SENSITIVE.YES, START.TAIL) == 16);
	* assert(t.indexOf('d', CASE_SENSITIVE.NO, START.TAIL) == 16);
	* assert(t.indexOf('1', CASE_SENSITIVE.NO, START.TAIL) == 0);
	* --------------------------------
	*/
	const int indexOf(C c, CASE_SENSITIVE CaseSensitive=CASE_SENSITIVE.YES, START Start=START.HEAD) {
		return hide_text.Find!(C)(data, [c], CaseSensitive, Start);
	}
	debug(text) unittest {
		_TEXT t;
		t = _TEXT("123456789abcdABCD");
		
		assert(t.indexOf('1', CASE_SENSITIVE.YES, START.HEAD) == 0);
		assert(t.indexOf('9', CASE_SENSITIVE.YES, START.HEAD) == 8);
		assert(t.indexOf('a', CASE_SENSITIVE.YES, START.HEAD) == 9);
		assert(t.indexOf('A', CASE_SENSITIVE.YES, START.HEAD) == 13);
		
		assert(t.indexOf('1', CASE_SENSITIVE.NO, START.HEAD) == 0);
		assert(t.indexOf('9', CASE_SENSITIVE.NO, START.HEAD) == 8);
		assert(t.indexOf('a', CASE_SENSITIVE.NO, START.HEAD) == 9);
		assert(t.indexOf('A', CASE_SENSITIVE.NO, START.HEAD) == 9);
		
		assert(t.indexOf('D', CASE_SENSITIVE.YES, START.TAIL) == 16);
		assert(t.indexOf('d', CASE_SENSITIVE.NO, START.TAIL) == 16);
		assert(t.indexOf('1', CASE_SENSITIVE.NO, START.TAIL) == 0);
	}
	/**
	文字検索。

	Params:
		Word = 検索する文字列。
	
		CaseSensitive = 大文字小文字の区別。
	
		Start = 検索位置。

	Returns:
		見つかった場合はそのインデックス、
		見つからなかった場合は-1を返す。

	Examples:
	* --------------------------------
	* Text t;
	* t = Text("123456789abcdABCD");
	* 
	* assert(t.indexOf(Text("1"), CASE_SENSITIVE.YES, START.HEAD) == 0);
	* assert(t.indexOf(Text("abcd"), CASE_SENSITIVE.YES, START.HEAD) == 9);
	* 
	* assert(t.indexOf(Text("1"), CASE_SENSITIVE.NO, START.HEAD) == 0);
	* assert(t.indexOf(Text("12"), CASE_SENSITIVE.NO, START.HEAD) == 0);
	* assert(t.indexOf(Text("ABCD"), CASE_SENSITIVE.NO, START.HEAD) == 9);
	* assert(t.indexOf(Text("123456789abcdABCD"), CASE_SENSITIVE.NO, START.HEAD) == 0);
	* assert(t.indexOf(Text("123456789abcdABCDE"), CASE_SENSITIVE.NO, START.HEAD) == -1);
	* 
	* assert(t.indexOf(Text("12"), CASE_SENSITIVE.YES, START.TAIL) == 0);
	* assert(t.indexOf(Text("23"), CASE_SENSITIVE.YES, START.TAIL) == 1);
	* assert(t.indexOf(Text("abcd"), CASE_SENSITIVE.YES, START.TAIL) == 9);
	* assert(t.indexOf(Text("ABCD"), CASE_SENSITIVE.NO, START.TAIL) == 13);
	* assert(t.indexOf(Text("abcd"), CASE_SENSITIVE.NO, START.TAIL) == 13);
	* --------------------------------

	*/
	const int indexOf(in _TEXT Word, CASE_SENSITIVE CaseSensitive=CASE_SENSITIVE.YES, START Start=START.HEAD) {
		return hide_text.Find!(C)(data, Word.data, CaseSensitive, Start);
	}
	debug(text) unittest {
		_TEXT t;
		t = _TEXT("123456789abcdABCD");
		
		assert(t.indexOf(_TEXT("1"), CASE_SENSITIVE.YES, START.HEAD) == 0);
		assert(t.indexOf(_TEXT("abcd"), CASE_SENSITIVE.YES, START.HEAD) == 9);
		
		assert(t.indexOf(_TEXT("1"), CASE_SENSITIVE.NO, START.HEAD) == 0);
		assert(t.indexOf(_TEXT("12"), CASE_SENSITIVE.NO, START.HEAD) == 0);
		assert(t.indexOf(_TEXT("ABCD"), CASE_SENSITIVE.NO, START.HEAD) == 9);
		assert(t.indexOf(_TEXT("123456789abcdABCD"), CASE_SENSITIVE.NO, START.HEAD) == 0);
		assert(t.indexOf(_TEXT("123456789abcdABCDE"), CASE_SENSITIVE.NO, START.HEAD) == -1);
		
		assert(t.indexOf(_TEXT("12"), CASE_SENSITIVE.YES, START.TAIL) == 0);
		assert(t.indexOf(_TEXT("23"), CASE_SENSITIVE.YES, START.TAIL) == 1);
		assert(t.indexOf(_TEXT("abcd"), CASE_SENSITIVE.YES, START.TAIL) == 9);
		assert(t.indexOf(_TEXT("ABCD"), CASE_SENSITIVE.NO, START.TAIL) == 13);
		assert(t.indexOf(_TEXT("abcd"), CASE_SENSITIVE.NO, START.TAIL) == 13);
	}
	
	/**
	文字列分割。

	Examples:
	* --------------------------------
	* Text t;
	* Text[] list;
	*
	* t = Text("123 456 789");
	* list = t.split(' ');
	* assert(list.length == 3);
	* assert(list[0].data == "123");
	* assert(list[1].data == "456");
	* assert(list[2].data == "789");
	*
	* t = Text("123,456,789,");
	* list = t.split(',');
	* assert(list.length == 4);
	* assert(list[0].data == "123");
	* assert(list[1].data == "456");
	* assert(list[2].data == "789");
	* assert(list[3].data == "");
	*
	* t = Text("");
	* list = t.split(',');
	* assert(!list.length);
	*
	* t = Text("123456789");
	* list = t.split(',');
	* assert(!list.length);
	*
	* t = Text("あ＠い＠う＠え＠お");
	* list = t.split('＠');
	* assert(list.length == 5);
	* 
	* --------------------------------
	
	
	History:
		$(HIST_VER 1.00β14,
			$(HIST_MSG 分割が変だったのを修正。)
		)
	*/
	const _TEXT[] split(in C c) {
		immutable Count=count(c);
		if(!Count) {
			return null;
		}
		/+
		auto list = new _TEXT[Count+1];
		for(auto i=0, j=0, k=0; i < data.length; i++) {
			if(data[i] == c) {
				list[k++] = _TEXT(data[j..i]);
				j = i + 1;
			} else if(k == Count) {
				// 最後
				list[k] = _TEXT(data[j..$]);
				break;
			}
		}
		return list;
		+/
		
		auto array=new _TEXT[Count+1];
		size_t i;
		foreach(data; std.algorithm.splitter(cast(S)data, c)) {
			array[i++] = _TEXT(data);
		}
		return array;
	}
	debug(text) unittest {
		_TEXT t;
		_TEXT[] list;
		
		t = _TEXT("123 456 789");
		list = t.split(' ');
		assert(list.length == 3);
		assert(list[0].data == "123");
		assert(list[1].data == "456");
		assert(list[2].data == "789");
		
		t = _TEXT("123,456,789,");
		list = t.split(',');
		assert(list.length == 4);
		assert(list[0].data == "123");
		assert(list[1].data == "456");
		assert(list[2].data == "789");
		assert(list[3].data == "");
		
		t = _TEXT("");
		list = t.split(',');
		assert(!list.length);
		
		t = _TEXT("123456789");
		list = t.split(',');
		assert(!list.length);
		
		static if(C.sizeof > '＠'.sizeof) {
			t = _TEXT("あ＠い＠う＠え＠お");
			list = t.split('＠');
			assert(list.length == 5);
		}
		
	}
	/**
	文字列分割。

	Examples:
	* --------------------------------
	* Text t;
	* Text[] list;
	*
	* t = Text("123..456..789");
	* list = t.split(Text(".."));
	* assert(list.length == 3);
	* assert(list[0].data == "123");
	* assert(list[1].data == "456");
	* assert(list[2].data == "789");
	*
	* t = Text("123....456..789");
	* list = t.split(Text(".."));
	* assert(list.length ==4);
	* assert(list[0].data == "123");
	* assert(list[1].data == "");
	* assert(list[2].data == "456");
	* assert(list[3].data == "789");
	*
	* t = Text("");
	* list = t.split(Text(".."));
	* assert(!list.length);
	*
	* t = Text("テキスト文字列data文字列てきすと");
	* list = t.split(Text("文字列"));
	* assert(list.length == 3);
	* assert(list[0].data == "テキスト");
	* assert(list[1].data == "data");
	* assert(list[2].data == "てきすと");
	*
	* t = Text("abcdefg");
	* list = t.split(Text(".."));
	* assert(!list.length);
	* --------------------------------

	History:
		$(HIST_VER 1.100,
			$(HIST_S 属性変更。)
			$(HIST_B 保持している文字列長が0でRange violation。)
			$(HIST_B 一致しない場合に戻り値がある。)
		)
		$(HIST_VER 1.00β14,
			$(HIST_NEW)
		)
	*/
	const _TEXT[] split(in _TEXT Word)
	out(r) {
		if(auto Count=count(Word)) {
			assert(Count+1 == r.length, Word.toANSI);
		}
	}
	body {
		if(!data.length) {
			return null;
		}
		immutable Count=count(Word);
		if(!Count) {
			return null;
		}
		
		immutable WordLength = Word.length;

		if(WordLength == 1) {
			return split(Word[0]);
		}

		size_t LoopLast;
		auto app=appender!(_TEXT[]);
		for(auto i=0, j=0; i < data.length - WordLength+1; i++) {
			if(data[i .. i + WordLength] == Word.data) {
				if(j == i) {
					app.put(_TEXT.init);
				} else {
					app.put(_TEXT(data[j..i]));
				}
				j = i + WordLength;
				i += WordLength-1;
				LoopLast++;
			}
			if(LoopLast == Count) {
				//app.put(cast(S)data[j..$]);
				app.put(_TEXT(data[j..$]));
				break;
			}
		}

		return app.data;
	}
	debug(text) unittest {
		_TEXT t;
		_TEXT[] list;
		t = _TEXT("123..456..789");
		list = t.split(_TEXT(".."));
		assert(list.length == 3);
		assert(list[0].data == "123");
		assert(list[1].data == "456");
		assert(list[2].data == "789");
		
		t = _TEXT("123....456..789");
		list = t.split(_TEXT(".."));
		assert(list.length ==4);
		assert(list[0].data == "123");
		assert(list[1].data == "");
		assert(list[2].data == "456");
		assert(list[3].data == "789");
		
		t = _TEXT("");
		list = t.split(_TEXT(".."));
		assert(!list.length);
		
		t = _TEXT("テキスト文字列data文字列てきすと");
		list = t.split(_TEXT("文字列"));
		assert(list.length == 3, _TEXT(list.length).toString);
		assert(list[0].data == "テキスト");
		assert(list[1].data == "data");
		assert(list[2].data == "てきすと");
		
		t = _TEXT("abcdefg");
		list = t.split(_TEXT(".."));
		assert(!list.length);
	}
	
	/**
	改行分割。

	Note:
		std.string.splitlines。
	
	History:
		$(HIST_VER 1.110,
			$(HIST_MSG 名前, splitlines, splitLines)
		)
		$(HIST_VER 1.100,
			$(HIST_NEW)
		)
	*/
	const _TEXT[] splitLines() {
		return toArray(std.string.splitlines(data));
	}

	/**
	空白分割。

	History:
		$(HIST_VER 1.110,
			$(HIST_NEW)
		)
	*/
	const _TEXT[] splitWhites()
	out(r) {
		foreach(t; r) {
			assert(t == t.strip);
		}
	}
	body {
		if(!data.length) {
			return null;
		}
		auto s=strip.data;
		if(!s.length) {
			return null;
		}

		auto app=appender!(S[]);
		size_t Start;
		bool Now=true;
		for(auto i=0; i < s.length; i++) {
			switch(s[i]) {
				case ' ', '\t', '\r', '\n', '\f', '\v':
					if(Now) {
						app.put(cast(S)s[Start .. i]);
						Now = false;
					}
					break;
				default:
					if(!Now) {
						Start = i;
						Now = true;
					}
			}
		}
		if(Now) {
			app.put(cast(S)s[Start .. $]);
		}
		return toArray(app.data);
	}
	debug(text) unittest {
		_TEXT t;
		_TEXT[] list;
		
		t = _TEXT("a b c");
		list = t.splitWhites();
		assert(list.length == 3);
		assert(list[0] == _TEXT("a"));
		assert(list[1] == _TEXT("b"));
		assert(list[2] == _TEXT("c"));
		
		t = _TEXT("a	b	c");
		list = t.splitWhites();
		assert(list.length == 3);
		assert(list[0] == _TEXT("a"));
		assert(list[1] == _TEXT("b"));
		assert(list[2] == _TEXT("c"));
		
		t = _TEXT("a\rb\nc");
		list = t.splitWhites();
		assert(list.length == 3);
		assert(list[0] == _TEXT("a"));
		assert(list[1] == _TEXT("b"));
		assert(list[2] == _TEXT("c"));
		
		t = _TEXT(" a 	   b\r   \n	c    	   ");
		list = t.splitWhites();
		assert(list.length == 3);
		assert(list[0] == _TEXT("a"));
		assert(list[1] == _TEXT("b"));
		assert(list[2] == _TEXT("c"));
		
		t = _TEXT("abc");
		list = t.splitWhites();
		assert(list.length == 1);
		assert(list[0] == _TEXT("abc"));
	}

	/**
	文字列置き換え。

	Textそのものに作用。
	
	History:
		$(HIST_VER 1.100,
			$(HIST_NEW)
		)
	*/
	ref _TEXT replaceRaw(in _TEXT SrcWord, in _TEXT NewWord) {
		if(!SrcWord.length || SrcWord == NewWord) {
			return this;
		}

		for(auto i=0; i < data.length; ) {
			//auto FindIndex=this[i .. this.length].find(SrcWord);
			auto FindIndex=this[i .. this.length].indexOf(SrcWord, CASE_SENSITIVE.YES, START.HEAD);
			if(FindIndex == -1) {
				// もう見つかんない
				return this;
			}
			if(NewWord.length) {
				//std.array.replace(data, FindIndex+i, FindIndex + i + SrcWord.length, cast(wchar[])NewWord.data);
				data = data[0 .. FindIndex+i] ~ NewWord.data ~ data[FindIndex + i + SrcWord.length .. $];
				i += FindIndex + NewWord.length;
			} else {
				data = data[0 .. FindIndex+i] ~ data[FindIndex + i + SrcWord.length .. $];
				i += FindIndex + SrcWord.length;
			}
		}

		return this;
	}
	debug(text) unittest {
		auto t1 = _TEXT("abcdefghi");
		t1.replaceRaw(_TEXT("b"), _TEXT("B"));
		assert(t1.data == "aBcdefghi", t1.toString);
		t1.replaceRaw(_TEXT("aBc"), _TEXT("ABC"));
		assert(t1.data == "ABCdefghi", t1.toString);
		t1 = _TEXT("abc abc abc");
		t1.replaceRaw(_TEXT("a"), _TEXT("A"));
		assert(t1.data == "Abc Abc Abc", t1.toString);
		t1.replaceRaw(_TEXT(" "), _TEXT(""));
		assert(t1.data == "AbcAbcAbc", t1.toString);
		t1.replaceRaw(_TEXT("Abc"), _TEXT("[ABC]"));
		assert(t1.data == "[ABC][ABC][ABC]", t1.toString);
	}
	/**
	文字列置き換え。
	
	History:
		$(HIST_VER 1.100,
			$(HIST_NEW)
		)
	*/
	const ref _TEXT replace(in _TEXT SrcWord, in _TEXT NewWord) {
		auto Result_TEXT=dup;
		return Result_TEXT.replaceRaw(SrcWord, NewWord);
	}
	debug(text) unittest {
		auto t1=_TEXT("abcdefghi");
		auto t2 = _TEXT("def");
		auto t3 = _TEXT("DEF");
		assert(t1.replace(t2, t3) == _TEXT("abcDEFghi"));
		assert(t1.replace(_TEXT("def"), t3) == _TEXT("abcDEFghi"));
		assert(t1.replace(_TEXT("def"), _TEXT("")) == _TEXT("abcghi"));
	}


	/**
	文字列挿入。
	
	Textそのものに作用。
	
	History:
		$(HIST_VER 1.100,
			$(HIST_NEW)
		)
	*/
	ref _TEXT insertRaw(in _TEXT Word, size_t Index) {
		std.array.insert(data, Index, Word.data);
		return this;
	}
	/**
	文字列挿入。

	History:
		$(HIST_VER 1.100,
			$(HIST_NEW)
		)
	*/
	const ref _TEXT insert(in _TEXT Word, size_t Index) {
		auto ResultText=dup;
		return ResultText.insertRaw(Word, Index);
	}
	debug(text) unittest {
		auto t=_TEXT("0123456789");
		assert(t.insert(_TEXT("abc"), 0) == _TEXT("abc0123456789"));
		assert(t.insert(_TEXT("abc"), 10) == _TEXT("0123456789abc"));
		assert(t.insert(_TEXT("abc"), 9) == _TEXT("012345678abc9"));
	}


	/**
	文字列を繰り返す。

	Params:
		Src = 対象の文字列。

		Repeat = 繰り返す回数。

	Returns:
		Src * Repeatの文字列。
	
	History:
		$(HIST_VER 1.100,
			$(HIST_P Repeatが一回の場合にSrcを返すのではなくSrcのコピーを返す用に変更。)
		)
		$(HIST_VER 1.00β14,
			$(HIST_NEW)
		)
	*/
	static ref _TEXT repeat(in _TEXT Src, size_t Repeat) {
		switch(Repeat) {
			case 0:
				return _TEXT.init;
			case 1:
				return Src.dup;
			default:
				auto TotalText=new _TEXT;

				TotalText.data.length = Src.length * Repeat;
				for(auto i=0; i < TotalText.data.length; i += Src.length) {
					TotalText.data[i .. i+Src.length] = Src.data;
				}

				return *TotalText;
		}
	}
	/**
	文字を繰り返す。

	Params:
		c = 繰り返す文字。

		Repeat = 繰り返す回数。

	Returns:
		c * Repeatの文字列。
	
	History:
		$(HIST_VER 1.100,
			$(HIST_NEW)
		)
	*/
	static ref _TEXT repeat(tchar c, size_t Repeat) {
		return repeat(_TEXT(c), Repeat);
	}

	/**
	現在の文字列を繰り返す。
	
	Params:
		Repeat = 繰り返す回数。

	Returns:
		現在の文字列 * Repeatの文字列。
	
	History:
		$(HIST_VER 1.050,
			$(HIST_NEW)
		)
	*/
	const ref _TEXT repeat(size_t Repeat) {
		return repeat(this, Repeat);
	}
	debug(text) unittest {
		assert(_TEXT.repeat(_TEXT('-'), 10).data == "----------");
		assert(_TEXT.repeat(_TEXT('-'), 0).data == "");
		assert(_TEXT.repeat(_TEXT('-'), 1).data == "-");
		assert(_TEXT.repeat('-', 10).data == "----------");
		assert(_TEXT.repeat('あ', 4).data == "ああああ");
		_TEXT t;
		t = _TEXT("abc");
		assert(_TEXT.repeat(t, 0) == _TEXT(``));
		assert(_TEXT.repeat(t, 1) == t);
		auto t2=_TEXT.repeat(t, 1);
		assert(t == t2);
		assert(t.data !is t2.data);
		t = _TEXT("abc");
		assert(t.repeat(2).data == "abcabc");
	}


	/**
	*/
	const ref _TEXT quot(C Quot) {
		return quot([Quot]);
	}
	/**
	*/
	const ref _TEXT quot(S Quot) {
		return quot(Quot, Quot);
	}
	/**
	*/
	const ref _TEXT quot(C left, C right) {
		return quot([left], [right]);
	}
	/**
	*/
	const ref _TEXT quot(S left, S right) {
		return _TEXT(left ~ data ~ right);
	}

	/**
	文字列は数値か。

	Params:
		Separator = セパレータを無視するか。

	Note:
		std.string.isNumeric。
	*/
	const bool isNumeric(bool Separator=true) {
		return std.string.isNumeric(strip._toString, Separator);
	}

	/**
	該当文字を省く。

	Params:
		Characters = フィルタとする文字。

	Returns:
		Charactersを取り除いたText。
	*/
	const ref _TEXT filter(in A Characters) {
		return _TEXT.wrap(hide_text.Filtering!(C)(data, Characters, hide_text.FILTERING.FILTER));
	}
	/// ditto
	const ref _TEXT filter(C Character, in A Characters ...) {
		return filter(Character ~ Characters);
	}
	debug(text) unittest {
		assert(_TEXT("a_b_c").filter(['_']) == _TEXT("abc"));
		assert(_TEXT("a b c").filter([' ']) == _TEXT("abc"));
		assert(_TEXT(" _ _ ").filter(['_', ' ']) == _TEXT(""));
		assert(_TEXT(" _ _ ").filter('_', ' ') == _TEXT(""));
		assert(_TEXT(" _ _ ").filter('_') == _TEXT("   "));
	}
	/**
	該当文字の抽出。

	*/
	const ref _TEXT extract(in A Characters) {
		return _TEXT.wrap(hide_text.Filtering!(C)(data, Characters, hide_text.FILTERING.EXTRACT));
	}
	/// ditto
	const ref _TEXT extract(C Character, in A Characters ...) {
		return extract(Character ~ Characters);
	}
	debug(text) unittest {
		assert(_TEXT("a_b_c").extract(['_']) == _TEXT("__"));
	}

	/**
	先頭文字列比較。

	文字列の先頭が指定した文字列で始まっているかを調べる。

	Params:
		Word = 比較したい文字列。
	
	History:
		$(HIST_VER 1.100,
			$(HIST_NEW)
		)
	*/
	const bool startsWith(in _TEXT Word) {
		if(!Word.length) {
			return false;
		}
		return std.algorithm.startsWith(data, Word.data) == 1;
	}
	debug(text) unittest {
		auto t=_TEXT("abcdef");
		assert(t.startsWith(_TEXT("abc")));
		assert(t.startsWith(_TEXT("abcdef")));
		assert(!t.startsWith(_TEXT("")));
		assert(!t.startsWith(_TEXT("Abcdefg")));
		assert(!t.startsWith(_TEXT("abcdefg")));
	}
	
	/**
	末尾文字列比較。

	文字列の末尾が指定した文字列で終わっているか調べる。

	Params:
		Word = 比較したい文字列。
	
	History:
		$(HIST_VER 1.100,
			$(HIST_NEW)
		)
	*/
	const bool endsWith(in _TEXT Word) {
		/+
		if(!Word.length) {
			return false;
		}
		if(length < Word.length) {
			return false;
		}

		return text[$-Word.length .. $] == Word.data;
		+/
		if(!Word.length) {
			return false;
		}
		return std.algorithm.endsWith(data, Word.data) == 1;
	}
	debug(text) unittest {
		auto t=_TEXT("abcdef");
		assert(t.endsWith(_TEXT("def")));
		assert(t.endsWith(_TEXT("abcdef")));
		assert(!t.endsWith(_TEXT("")));
		assert(!t.endsWith(_TEXT("abcdefg")));
	}

	/**
	*/
	static ref _TEXT join(in _TEXT[] texts, in _TEXT sep) {
		immutable TextsLength = texts.length;

		if(!TextsLength) {
			return _TEXT.init;//Text("");//Text.emptyText();
		}

		auto ret=new _TEXT;
		auto SepLength=sep.length;

		size_t TotalLength;
		for(auto i=0; i < TextsLength; i++) {
			TotalLength += texts[i].length;
		}
		TotalLength += (SepLength * (TextsLength - 1));

		ret.data.length = TotalLength;
		size_t StartPos;

		for(auto i=0; i < TextsLength; i++) {
			if(i && SepLength) {
				ret.data[StartPos .. StartPos+SepLength] = sep.data;
				StartPos += SepLength;
			}
			if(auto TextLength=texts[i].length) {
				ret.data[StartPos .. StartPos+TextLength] = texts[i].data.dup;
				StartPos += texts[i].length;
			}
		}

		return *ret;
	}
	debug(text) unittest {
		assert(_TEXT.join(_TEXT.toArray("aaa", "bbb", "ccc"), (_TEXT(""))).data =="aaabbbccc");
		assert(_TEXT.join(_TEXT.toArray("aaa", "bbb", "ccc"), (_TEXT("+"))).data =="aaa+bbb+ccc");
		assert(_TEXT.join(_TEXT.toArray("", "bbb", ""), (_TEXT("*"))).data =="*bbb*");
		assert(_TEXT.join(_TEXT.toArray("", "", "", ""), (_TEXT("*"))).data =="***");
	}

	/**
	History:
		$(HIST_VER 1.120,
			$(HIST_CHG 所属, etc.negui.system, etc.negui.system._NTEXT)
			$(HIST_CHG 名前, GetUnique, getUnique)
		)
		$(HIST_VER 1.100,
			$(HIST_CHG 名前, GetNoneDouble, GetUnique)
		)
		$(HIST_VER 1.090,
			$(HIST_MSG 新規作成というかnemuxi.gui.window.dialog.settingdialog.groupdialogから独立。)
		)
	*/
	static ref _TEXT getUnique(in _TEXT[] TextArray, in _TEXT Src)
	in {
		assert(TextArray.length);
		assert(Src.length);
	}
	body {
		scope ChangeName=Src.dup;
		size_t n=1;
		for(auto i=0; i < TextArray.length; i++) {
			if(TextArray[i] == ChangeName) {
				ChangeName = _TEXT("%s(%s)", Src, ++n);
				i=-1;
				continue;
			}
		}
		return ChangeName.dup;
	}
	debug(text) unittest {
		auto ts=toArray(
			"abc",
			"def",
			"ABC",
			"DEF",
			"123",
			"456",
			"あ",
			"あ(1)",
			"あ(2)",
			"あ(3)"
		);
		assert(getUnique(ts, _TEXT("abc")) == _TEXT("abc(2)"));
		assert(getUnique(ts, _TEXT("def")) == _TEXT("def(2)"));
		assert(getUnique(ts, _TEXT("XYZ")) == _TEXT("XYZ"));
		assert(getUnique(ts, _TEXT("あ")) == _TEXT("あ(4)"));
	}




	
	
}

alias _NTEXT!( char) CTEXT; ///
alias _NTEXT!(wchar) WTEXT; ///
alias _NTEXT!(dchar) DTEXT; ///
alias _NTEXT!(tchar) Text;  ///

///
interface IText {
	const Text toText();
}
/**
*/
interface ITitleText {
	/**
	*/
	const Text text();
	/**
	*/
	void text(in Text);
}


///
template IsNTEXT(T) {
	enum bool IsNTEXT = staticIndexOf!(Unqual!(T), CTEXT, WTEXT, DTEXT) != -1;
}
///
template IsText(T) {
	enum bool IsText = is(Unqual!(T) == Text);
}

/**

NULL結合+終端NULL二つの文字列郡を作成。
Windowsへの橋渡し用。

Bugs:
	汎用性を持たせるべき。

ToDo:
	突貫工事。

History:
	$(HIST_VER 1.100,
		$(HIST_NEW)
	)
*/
struct TEXTNULLLIST {
	Text[] datas;
	alias datas this;

	static ref TEXTNULLLIST opCall(in tchar* Data) {
		auto TextNullList=new TEXTNULLLIST;

		auto p=Data;
		size_t i, j;
		do {
			if(!p[i]) {
				// 落ち着いたら対処する。
				TextNullList.datas ~= Text(p[j]);
				j = ++i;
			}
		} while(p[i++]);

		return *TextNullList;
	}
	debug(text) unittest {
		auto tnl=TEXTNULLLIST("abc\0def\0efg\0\0"w.ptr);
		assert(tnl.length == 3, Text(tnl.length).toString);
	}

	const size_t dataLength() {
		size_t Length=datas.length+1;

		foreach(ref const(Text) data; datas) {
			Length += data.length;
		}

		return Length;
	}

	const void* ptr() {
		if(!datas.length) {
			return null;
		}

		size_t Length=dataLength;
		auto DataList=new wchar[Length];
		for(auto i=0,j=0; i < Length-1; j++) {
			auto Size=i + datas[j].length;
			DataList[i .. Size] = datas[j].data;
			i = Size;
			DataList[i++] = 0;
		}
		DataList[$-1] = 0;

		return cast(void*)DataList;
	}
}

