﻿/**
タイマーというか時間関係。

std.timeがあまりにもあれだったんで置き換え。


*/
module etc.negui.system.time;

debug import std.stdio: wl = writefln;
debug(time) void main() {}

import std.array;
public import std.date;
import std.contracts;
import std.conv;

import win32.windows;

public import etc.negui.system.text;
public import etc.negui.system.exception;
public import etc.negui.system.error;
public import etc.negui.system.raii;
//mport etc.negui.system.error;
import etc.negui.system.meta.memberproperty;
import etc.negui.system.utility.cmp;
import etc.negui.system.utility.number;

/**
このモジュールで発行される例外の親玉。

History:
	$(HIST_VER 1.120,
		$(HIST_S typedefから継承へ。)
	)
	$(HIST_VER 1.110,
		$(HIST_S 暫定的に継承からtypedefへ。)
	)
*/
class TimeException: NeGuiException {
	mixin MixInNeGuiException;
}

/**
FILEDATETIMEに関する例外。

History:
	$(HIST_VER 1.120,
		$(HIST_S typedefから継承へ。)
	)
	$(HIST_VER 1.110,
		$(HIST_S 暫定的に継承からtypedefへ。)
	)
*/
class FileDateTimeException: TimeException {
	mixin MixInNeGuiException;
}


/**
ファイル時刻。

FILETIMEのラッパー。

History:
	$(HIST_VER 1.100,
		$(HIST_NEW)
	)
*/
struct FILEDATETIME {
	/// FileTimeとfullは共用。
	union {
		/// 実体。
		FILETIME FileTime;
		/// なんとなく
		long full;
	}
	mixin(SMixInStructHiddenOriginal!(FILETIME)(q{FileTime}));

	
	mixin(SMixInStructGetSet!(DWORD)("low", q{FileTime.dwLowDateTime}));
	mixin(SMixInStructGetSet!(DWORD)("high", q{FileTime.dwHighDateTime}));
	//mixin(SMixInStructGetSet!(long)("full", q{FullTime}));


	/**
	ファイル時刻->システム日時形式変換。

	Params:
		FileDateTime = 変換したいファイル時刻。

	Returns:
		変換されたシステム日時形式。

	Throws:
		変換失敗時にFileDateTimeException。

	History:
		$(HIST_VER 1.110,
			$(HIST_P 例外発行時のメッセージをERRで出力。)
		)
	*/
	static ref SYSTEMDATETIME toSystemDateTime(const ref FILEDATETIME FileDateTime) {
		auto SystemDateTime=new SYSTEMDATETIME;
		
		//enforce(FileTimeToSystemTime(cast(FILETIME*)&FileDateTime, SystemDateTime.ptr), new FileDateTimeException(Text("toSystemDateTime")));
		enforce(FileTimeToSystemTime(cast(FILETIME*)&FileDateTime, SystemDateTime.ptr), new FileDateTimeException(ERR.toText));
		
		return *SystemDateTime;
	}
	/**
	システム日時形式変換。

	Returns:
		自身の情報を元に変換されたシステム日時形式。
	
	Throws:
		変換失敗時にFileDateTimeException。
	*/
	const ref SYSTEMDATETIME toSystemDateTime() {
		return toSystemDateTime(this);
	}

	/**
	UTCファイル時刻->ローカルファイル時刻変換。

	Params:
		FileDateTime = 変換したいUTCファイル時刻情報。

	Returns:
		変換されたローカルファイル時刻。

	Throws:
		変換失敗時にFileDateTimeException。
	
	History:
		$(HIST_VER 1.110,
			$(HIST_P 例外発行時のメッセージをERRで出力。)
		)
	*/
	static ref FILEDATETIME toLocalFileDateTime(const ref FILEDATETIME FileDateTime) {
		auto LocalFileDateTime=new FILEDATETIME;
		
		//enforce(FileTimeToLocalFileTime(FileDateTime.ptr, LocalFileDateTime.ptr), new FileDateTimeException(Text("toLocalFileDateTime")));
		enforce(FileTimeToLocalFileTime(FileDateTime.ptr, LocalFileDateTime.ptr), new FileDateTimeException(ERR.toText));

		return *LocalFileDateTime;
	}
	/**
	ローカルファイル時刻変換。

	Returns:
		自身の情報を元に変換されたローカルファイル時刻。
	
	Throws:
		変換失敗時にFileDateTimeException。
	*/
	const ref FILEDATETIME toLocalFileDateTime() {
		return toLocalFileDateTime(this);
	}

	/**
	ローカルファイル時刻->UTCファイル時刻変換。
	
	Returns:
		変換されたUTCファイル時刻。
	
	Throws:
		変換失敗時にFileDateTimeException。
	
	History:
		$(HIST_VER 1.110,
			$(HIST_P 例外発行時のメッセージをERRで出力。)
		)
	*/
	static ref FILEDATETIME toSystemFileDateTime(const ref FILEDATETIME FileDateTime) {
		auto SystemFileDateTime=new FILEDATETIME;
		
		//enforce(LocalFileTimeToFileTime(FileDateTime.ptr, SystemFileDateTime.ptr), new FileDateTimeException(Text("toSystemFileDateTime")));
		enforce(LocalFileTimeToFileTime(FileDateTime.ptr, SystemFileDateTime.ptr), new FileDateTimeException(ERR.toText));

		return *SystemFileDateTime;
	
	}
	/**
	UTCファイル時刻変換。

	Returns:
		自身の情報を元に変換されたUTCファイル時刻。
	
	Throws:
		変換失敗時にFileDateTimeException。
	*/
	const ref FILEDATETIME toSystemFileDateTime() {
		return toSystemFileDateTime(this);
	}
	
	/***/
	const bool opEquals(const ref FILEDATETIME FileDateTime) {
		return full == FileDateTime.full;
	}
	/***/
	const int opCmp(const ref FILEDATETIME FileDateTime) {
		return CompareFileTime(this.ptr, FileDateTime.ptr);
	}
}
/**
SYSTEMDATETIMEに関する例外。

History:
	$(HIST_VER 1.120,
		$(HIST_S typedefから継承へ。)
	)
	$(HIST_VER 1.110,
		$(HIST_S 暫定的に継承からtypedefへ。)
	)
	$(HIST_VER 1.100,
		$(HIST_NEW)
	)
*/
class SystemDateTimeException: TimeException {
	mixin MixInNeGuiException;
}

/**
システム日時。

SYSTEMTIMEのラッパー。

History:
	$(HIST_VER 1.100,
		$(HIST_NEW)
	)
*/
struct SYSTEMDATETIME {
	/// 実体。
	SYSTEMTIME SystemTime;
	mixin(SMixInStructHiddenOriginal!(SYSTEMTIME)(q{SystemTime}));

	mixin(SMixInStructGetSet!(WORD)("year",   q{SystemTime.wYear}));
	mixin(SMixInStructGetSet!(WORD)("month",  q{SystemTime.wMonth}));
	mixin(SMixInStructGetSet!(WORD)("day",    q{SystemTime.wDay}));
	mixin(SMixInStructGetSet!(WORD)("week",   q{SystemTime.wDayOfWeek}));
	mixin(SMixInStructGetSet!(WORD)("hour",   q{SystemTime.wHour}));
	mixin(SMixInStructGetSet!(WORD)("minute", q{SystemTime.wMinute}));
	mixin(SMixInStructGetSet!(WORD)("second", q{SystemTime.wSecond}));
	mixin(SMixInStructGetSet!(WORD)("ms",     q{SystemTime.wMilliseconds}));

	/**
	システム日時->ファイル時刻変換。

	Params:
		SystemDateTime = 変換したいシステム日時情報。

	Returns:
		変換されたファイル時刻。

	Throws:
		変換失敗時にSystemDateTimeException。
	
	History:
		$(HIST_VER 1.110,
			$(HIST_P 例外発行時のメッセージをERRで出力。)
		)
	*/
	const ref FILEDATETIME toFileDateTime(const ref SYSTEMDATETIME SystemDateTime) {
		auto FileDateTime=new FILEDATETIME;

		//enforce(SystemTimeToFileTime(cast(SYSTEMTIME*)SystemDateTime.ptr, FileDateTime.ptr), new SystemDateTimeException(Text("toFileDateTime")));
		enforce(SystemTimeToFileTime(cast(SYSTEMTIME*)SystemDateTime.ptr, FileDateTime.ptr), new SystemDateTimeException(ERR.toText));

		return *FileDateTime;
	}
	/**
	ファイル時刻変換。

	Returns:
		自身の情報を元に変換された変換されたファイル時刻。

	Throws:
		変換失敗時にSystemDateTimeException。
	*/
	const ref FILEDATETIME toFileDateTime() {
		return toFileDateTime(this);
	}

	/**
	現在のローカル日時を取得。

	Returns:
		現在のローカル日時情報。
	*/
	static ref SYSTEMDATETIME localTime() {
		auto SystemDateTime=new SYSTEMDATETIME;
		
		GetLocalTime(SystemDateTime.ptr);

		return *SystemDateTime;
	}

	
	/***/
	const bool opEquals(const ref SYSTEMDATETIME SystemDateTime) {
		return toFileDateTime == SystemDateTime.toFileDateTime;
	}
	/***/
	const int opCmp(const ref SYSTEMDATETIME SystemDateTime) {
		return Cmp(toFileDateTime, SystemDateTime.toFileDateTime);
	}

	/***/
	const Text toText() {
		return Text(
			"%s-%02s-%02sT%02s:%02s:%02s.%03s",
			SystemTime.wYear,
			SystemTime.wMonth,
			SystemTime.wDay,
			SystemTime.wHour,
			SystemTime.wMinute,
			SystemTime.wSecond,
			SystemTime.wMilliseconds
		);
	}
	/***/
	const string toString() {
		return toText.toString;
	}
	/// ditto
	string toString() {
		return toText.toString;
	}

	/// 最小/最大値
	enum RANGE: WORD {
		MONTH_MIN  = 1,   /// 月
		MONTH_MAX  = 12,  /// ditto
		DAY_MIN    = 1,   /// 日
		DAY_MAX    = 31,  /// ditto
		WEEK_MIN   = 0,   /// 曜日
		WEEK_MAX   = 6,   /// ditto
		HOUR_MIN   = 0,   /// 時
		HOUR_MAX   = 23,  /// ditto
		MINUTE_MIN = 0,   /// 分
		MINUTE_MAX = 59,  /// ditto
		SECOND_MIN = 0,   /// 秒
		SECOND_MAX = 59,  /// ditto
		MS_MIN     = 0,   /// ミリ秒
		MS_MAX     = 999, /// ditto
	}
	private static pure nothrow string IsRange(string Method, string RANGE) {
		return `
			static pure nothrow bool ` ~ Method ~ `(WORD Value) {
				return RANGE.` ~ RANGE ~ `_MIN <= Value && Value <= RANGE.` ~ RANGE ~ `_MAX;
			}
		`;
	}
	
	/**
	年として合法か。

	0年ならfalseとする。
	*/
	static pure nothrow bool isYear(WORD Year) {
		return Year != 0;
	}
	mixin(IsRange("isMonth",  "MONTH"));
	mixin(IsRange("isDay",    "DAY"));
	mixin(IsRange("isWeek",   "WEEK"));
	mixin(IsRange("isHour",   "HOUR"));
	mixin(IsRange("isMinute", "MINUTE"));
	mixin(IsRange("isSecond", "SECOND"));
	mixin(IsRange("isMs",     "MS"));

	/**
	閏年判定。

	Params:
		Year = 調べたい年。

	In:
		isYear(Year)
	*/
	static pure nothrow bool isLeapYear(WORD Year)
	in {
		assert(isYear(Year));
	}
	body {
		return Year % 4 == 0 && (Year % 100 || !(Year % 400));
	}
	debug(time) unittest {
		assert(!isLeapYear(1));
		assert(!isLeapYear(2));
		assert(!isLeapYear(3));
		assert(isLeapYear(4));
		assert(isLeapYear(1996));
		assert(isLeapYear(2000));
		assert(isLeapYear(2004));
		assert(!isLeapYear(2100));
	}

	/// 曜日。
	enum WEEK: WORD {
		SUNDAY = 0, /// 日曜日。
		MONDAY,     /// 月曜日。
		TUESDAY,    /// 火曜日。
		WEDNESDAY,  /// 水曜日。
		THURSDAY,   /// 木曜日。
		FRIDAY,     /// 金曜日。
		SATURDAY,   /// 土曜日。
	}
	/**
	曜日の取得。

	Returns:
		WEEK。

	Standards:
		Zellerの公式
	
	See_Also:
		$(LINK_TITLE http://ja.wikipedia.org/wiki/%E3%83%84%E3%82%A7%E3%83%A9%E3%83%BC%E3%81%AE%E5%85%AC%E5%BC%8F, ウィキペディア)

	In:
		isYear(Year) && isMonth(Month) && isDay(Day)
	*/
	static pure nothrow WEEK getWeekDay(in WORD Year, in WORD Month, in WORD Day)
	in {
		assert(isYear(Year));
		assert(isMonth(Month));
		assert(isDay(Day));
	}
	out(r) {
		assert(isWeek(r));
	}
	body {
		immutable MonthPrev = Month > 2;
		immutable YEAR  = MonthPrev ? Year:  Year - 1;
		immutable MONTH = MonthPrev ? Month: Month + 12;
		//return cast(WORD)((YEAR + floor(YEAR / 4) - floor(YEAR / 100) + floor(YEAR / 400) + floor(2.6 * MONTH + 1.6) + Day) % 7);
		return cast(WEEK)((YEAR + (YEAR / 4) - (YEAR / 100) + (YEAR / 400) + (2.6 * MONTH + 1.6) + Day) % 7);
	}
	debug(time) unittest {
		assert(getWeekDay(2000, 1, 1) == 6);
		assert(getWeekDay(2000, 1, 31) == 1);
		assert(getWeekDay(2000, 2, 1) == 2);
		assert(getWeekDay(2000, 2, 28) == 1);
		assert(getWeekDay(2000, 2, 29) == 2);
		assert(getWeekDay(2000, 3, 1) == 3);
	}

	/**
	月の日数。

	Params:
		Year = 調べたい年。

		Month = 調べたい月。
	
	In:
		isYear(Year) && isMonth(Month)
	*/
	static pure nothrow WORD getMonthDay(WORD Year, WORD Month)
	in {
		assert(isYear(Year));
		assert(isMonth(Month));
	}
	out(r) {
		//assert(isWeek(r));
	}
	body {
		final switch(Month) {
			case 1, 3, 5, 7, 8, 10, 12:
				return 31;
			case 4, 6, 9, 11:
				return 30;
			case 2:
				return isLeapYear(Year) ? 29: 28;
		}
	}
	debug(time) unittest {
		assert(getMonthDay(2000, 1) == 31);
		assert(getMonthDay(2000, 4) == 30);
		assert(getMonthDay(2000, 2) == 29);
		assert(getMonthDay(2001, 2) == 28);
	}

	/**
	次の月を取得。

	Params:
		NowMonth = 現在の月。

	Returns:
		来月。
	
	In:
		isMonth(NowMonth)
	*/
	static pure nothrow WORD nextMonth(WORD NowMonth)
	in {
		assert(isMonth(NowMonth));
	}
	body {
		if(NowMonth == 12) {
			return 1;
		} else {
			return cast(WORD)(NowMonth + 1);
		}
	}
	debug(time) unittest {
		WORD[] src =[1, 2, 3, 4, 5, 6, 7, 8,  9, 10, 11, 12];
		WORD[] next=[2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12,  1];
		for(WORD i=0; i < src.length; i++) {
			assert(nextMonth(src[i]) == next[i]);
		}
	}
	/**
	前の月を取得。

	Params:
		NowMonth = 現在の月。

	Returns:
		先月。
	
	In:
		isMonth(NowMonth)
	*/
	static pure nothrow WORD prevMonth(WORD NowMonth)
	in {
		assert(isMonth(NowMonth));
	}
	body {
		if(NowMonth == 1) {
			return 12;
		} else {
			return cast(WORD)(NowMonth - 1);
		}
	}
	debug(time) unittest {
		WORD[] next=[1, 2, 3, 4, 5, 6, 7, 8,  9, 10, 11, 12];
		WORD[] src =[2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12,  1];
		for(WORD i=0; i < src.length; i++) {
			assert(prevMonth(src[i]) == next[i]);
		}
	}
	/**
	次の日を取得。

	Params:
		Year  = 現在の年。
		Month = 現在の月。
		Day   = 現在の日。

	Returns:
		次の日。
	
	In:
		isYear(Year) && isMonth(Month) && isDay(Day)
	*/
	static pure nothrow WORD nextDay(WORD Year, WORD Month, WORD Day)
	in {
		assert(isYear(Year));
		assert(isMonth(Month));
		assert(isDay(Day));
	}
	body {
		// 月末判定
		immutable LastDay=getMonthDay(Year, Month);
		if(Day >= LastDay) {
			return 1;
		} else {
			return cast(WORD)(Day + 1);
		}
	}
	debug(time) unittest {
		assert(nextDay(2000, 1, 1) == 2);
		assert(nextDay(2000, 1, 31) == 1);
		assert(nextDay(2000, 2, 28) == 29);
		assert(nextDay(2000, 2, 29) == 1);
		assert(nextDay(2001, 2, 28) == 1);
	}
	/**
	前の日を取得。

	Params:
		Year  = 現在の年。
		Month = 現在の月。
		Day   = 現在の日。

	Returns:
		前の日。
	
	In:
		isYear(Year) && isMonth(Month) && isDay(Day)
	*/
	static pure nothrow WORD prevDay(WORD Year, WORD Month, WORD Day)
	in {
		assert(isYear(Year));
		assert(isMonth(Month));
		assert(isDay(Day));
	}
	body {
		// 月初判定
		if(Day == 1) {
			return getMonthDay(Year, prevMonth(Month));
		} else {
			return cast(WORD)(Day - 1);
		}
	}
	debug(time) unittest {
		assert(prevDay(2000, 12, 1) == 30);
		assert(prevDay(2000, 1, 31) == 30);
		assert(prevDay(2000, 2, 29) == 28);
		assert(prevDay(2000, 3, 1) == 29);
		assert(prevDay(2001, 3, 1) == 28);
	}

	/**
	午前・午後。
	
	History:
		$(HIST_VER 1.110,
			$(HIST_NEW)
		)
	*/
	enum MERIDIEM {
		AM, /// 
		PM, /// 
	}
	/**
	時間から午前・午後の取得。

	Params:
		Hour = 調べる時間。

	Returns:
		MERIDIEM。
	
	In:
		isHour(Hour)
	
	History:
		$(HIST_VER 1.110,
			$(HIST_NEW)
		)
	*/
	static pure nothrow MERIDIEM getMeridiem(WORD Hour)
	in {
		assert(isHour(Hour));
	}
	body {
		if(0 <= Hour && Hour < 12) {
			return MERIDIEM.AM;
		}
		return MERIDIEM.PM;
	}
	/**
	時間を午前午後に変換。
	
	Returns:
		変換された値。
	
	In:
		isHour(Hour)
	
	History:
		$(HIST_VER 1.110,
			$(HIST_NEW)
		)
	*/
	static pure nothrow WORD getMeridiemHour(WORD Hour)
	in {
		assert(isHour(Hour));
	}
	body {
		if(Hour >= 12) {
			return cast(WORD)(Hour - 12);
		}
		return Hour;
	}
	debug(time) unittest {
		assert(getMeridiemHour(0) == 0);
		assert(getMeridiemHour(11) == 11);
		assert(getMeridiemHour(12) == 0);
		assert(getMeridiemHour(23) == 11);
	}

}
debug(time) unittest {
	SYSTEMDATETIME s;
	s.year = 2000;
	s.month= 2;
	s.day= 2;
	s.second = 2;
	auto f=s.toFileDateTime();
	assert(f.full == f.toSystemDateTime.toFileDateTime.full);
}


/**
DateTimeに関する例外。


History:
	$(HIST_VER 1.120,
		$(HIST_S typedefから継承へ。)
	)
	$(HIST_VER 1.110,
		$(HIST_S 暫定的に継承からtypedefへ。)
	)
	$(HIST_VER 1.100,
		$(HIST_NEW)
	)

*/
class DateTimeException: SystemDateTimeException {
	mixin MixInNeGuiException;
}

/// DateTime(MODE, in Text)に関する例外。
final class DateTimeParseException: DateTimeException {
	private {
		const Text Source;
	}
	this(in Text msg, in Text Source, Throwable e=null, string _file=__FILE__, int _line=__LINE__) {
		this.Source = Source;
		super(msg, e, _file, _line);
	}
	override protected string ExceptionMessage() {
		return Text(
			"%s@%s(%s): Source -> %s, %s [%s]",
			this.classinfo.name,
			file,
			line,
			Source,
			msg,
			time
		).str;
	}
}

/**
日時を扱う専用クラス。

ﾈﾑぃに特化。

Bugs:
	調子乗りすぎた。
	もうしんどい、いやや。

	タイムゾーンに関しては知識がないから未実装だよ。
	

	Examples:
	* --------------------------------
	* // 現在時刻を元に生成
	* auto dt1=new DateTime();
	* // 一月末日に設定
	* dt1.month = 1;
	* dt1.day   = 31;
	* // 四月に設定
	* dt1.month = 4;
	* assert(dt1.day == 30); // 四月の末日は30日
	* // モード変更
	* dt1.mode = DateTime.MODE.STRICT;
	* // 一月末日に設定
	* dt1.month = 1;
	* dt1.day   = 31;
	* try {
	* 	// 四月に設定
	* 	dt1.month = 4;
	* 	assert(false);
	* } catch(DateTimeException) {
	* 	 // 四月の末日は30日なので例外が飛んでくる
	* 	assert(true);
	* }
	* 
	* // 指定時刻を元に生成
	* // FLEXIBLEで2010/1/32を生成
	* auto dt2=new DateTime(DateTime.MODE.FLEXIBLE,
	* 	2010, 1, 32
	* );
	* // 一月末の次の日で2/1
	* assert(dt2.month == 2);
	* assert(dt2.day   == 1);
	* // STRICTで2010/1/32を生成
	* try {
	* 	auto dt3=new DateTime(
	* 		DateTime.MODE.STRICT,
	* 		2010, 1, 32
	* 	);
	* 	assert(false);
	* } catch(DateTimeException) {
	* 	// STRICTだと怒られる。
	* 	assert(true);
	* }
	* 
	* // 文字列から生成
	* // YYYY-MM-DDThh:mm:ss.MS形式ならDateTimeに変更可能
	* with(new DateTime(DateTime.MODE.FLEXIBLE, Text("2010-01-02T03:04:05.678"))) {
	* 	assert(year   == 2010);
	* 	assert(month  == 1);
	* 	assert(day    == 2);
	* 	assert(hour   == 3);
	* 	assert(minute == 4);
	* 	assert(second == 5);
	* 	assert(ms     == 678);
	* }
	* // 限定的に生成することも可能
	* with(new DateTime(DateTime.MODE.FLEXIBLE, Text("2010-12"))) {
	* 	assert(year   == 2010);
	* 	assert(month  == 12);
	* 	assert(day    == 1);
	* 	assert(hour   == 0);
	* 	assert(minute == 0);
	* 	assert(second == 0);
	* 	assert(ms     == 0);
	* }
	* // FLEXIBLEで生成
	* with(new DateTime(DateTime.MODE.FLEXIBLE, Text("2010-12-33"))) {
	* 	assert(year   == 2011);
	* 	assert(month  == 1);
	* 	assert(day    == 2);
	* }
	* try {
	* 	// STRICTで生成
	* 	with(new DateTime(DateTime.MODE.STRICT, Text("2010-12-33"))) {
	* 		assert(false);
	* 	}
	* } catch(DateTimeException) {
	* 	// やっぱり怒られる
	* 	assert(true);
	* }
	* 
	* // 文字列化と書式とマスク
	* // まずは 2010-01-02T03:04:05.006 で生成
	* auto dt4=new DateTime(DateTime.MODE.FLEXIBLE, 2010, 1, 2, 3, 4, 5, 6);
	* // ミリ秒が無い
	* assert(dt4.toText == Text("2010-01-02T03:04:05"));
	* // DateTimeのデフォルト書式にそもそもミリ秒が無い
	* assert(dt4.format == Text("[YYYY]-[MM]-[DD]T[hh]:[mm]:[ss]"));
	* // 書式設定、ミリ秒を追加。
	* dt4.format ~= Text(".[ff]");
	* // 追加完了
	* assert(dt4.format == Text("[YYYY]-[MM]-[DD]T[hh]:[mm]:[ss].[ff]"));
	* // でもなんか変
	* assert(dt4.toText == Text("2010-01-02T03:04:05.[ff]"));
	* // そもそもデフォルトマスクにミリ秒指定が無い
	* assert((dt4.mask & DateTime.MASK.MS) != DateTime.MASK.MS);
	* // マスクを設定
	* dt4.mask |= DateTime.MASK.MS;
	* // 設定完了
	* assert((dt4.mask & DateTime.MASK.MS) == DateTime.MASK.MS);
	* // 文字列化
	* assert(dt4.toText == Text("2010-01-02T03:04:05.006"));
	* // 色々文字列化
	* // 下二桁の年
	* dt4.format = Text("[YY]");
	* assert(dt4.toText == Text("10"));
	* // 0で埋めない
	* dt4.format = Text("[Y]-[M]-[D]T[h]:[m]:[s].[f]");
	* assert(dt4.toText == Text("2010-1-2T3:4:5.6"));
	* // 曜日
	* dt4.mask |= DateTime.MASK.WEEK;
	* dt4.format = Text("[@$W]");
	* // 2010/1/2は土曜日
	* assert(dt4.toText == Text("Sat"));
	* assert(dt4.week   == SYSTEMDATETIME.WEEK.SATURDAY);
	* // 2010/1/3は日曜日
	* dt4.day = 3;
	* assert(dt4.toText == Text("Sun"));
	* assert(dt4.week   == SYSTEMDATETIME.WEEK.SUNDAY);
	* // モード変更, 2010/1/4は月曜日
	* dt4.mode = DateTime.MODE.STRICT;
	* dt4.day  = 4;
	* assert(dt4.toText == Text("Mon")); // STRICTでも曜日は勝手に変わる
	* assert(dt4.week   == SYSTEMDATETIME.WEEK.MONDAY);
	* // 曜日名の変更, 2010/1/5は火曜日
	* dt4.format = Text("[@W]");
	* dt4.weeks[SYSTEMDATETIME.WEEK.TUESDAY] = Text("火曜日");
	* dt4.day  = 5;
	* assert(dt4.toText == Text("火曜日"));
	* assert(dt4.week   == SYSTEMDATETIME.WEEK.TUESDAY);
	* // 月も数値じゃなくて名前でとってみる
	* dt4.format = Text("[@M]");
	* assert(dt4.toText == Text("January"));
	* 
	* // それっぽく日曜日
	* dt4.day = 3;
	* dt4.format = Text("[@$W] [@$M] [D] [hh]:[mm]:[ss] [YYYY]");
	* assert(dt4.toText == Text("Sun Jan 3 03:04:05 2010"));
	* --------------------------------

History:
	$(HIST_VER 1.110,
		$(HIST_S IInitializeを継承。)
	)
	$(HIST_VER 1.101,
		$(HIST_P weekの不変条件をmaskに関係なく行うように変更。)
	)
	$(HIST_VER 1.100,
		$(HIST_S 全面修正)
	)
	$(HIST_VER 1.00β19,
		$(HIST_S Timeを継承)
	)
*/
class DateTime: IText, IInitialize {
	debug(time) unittest {
		// 現在時刻を元に生成
		auto dt1=new DateTime();
		// 一月末日に設定
		dt1.month = 1;
		dt1.day   = 31;
		// 四月に設定
		dt1.month = 4;
		assert(dt1.day == 30); // 四月の末日は30日
		// モード変更
		dt1.mode = DateTime.MODE.STRICT;
		// 一月末日に設定
		dt1.month = 1;
		dt1.day   = 31;
		try {
			// 四月に設定
			dt1.month = 4;
			assert(false);
		} catch(DateTimeException) {
			 // 四月の末日は30日なので例外が飛んでくる
			assert(true);
		}
		
		// 指定時刻を元に生成
		// FLEXIBLEで2010/1/32を生成
		auto dt2=new DateTime(DateTime.MODE.FLEXIBLE,
			2010, 1, 32
		);
		// 一月末の次の日で2/1
		assert(dt2.month == 2);
		assert(dt2.day   == 1);
		// STRICTで2010/1/32を生成
		try {
			auto dt3=new DateTime(
				DateTime.MODE.STRICT,
				2010, 1, 32
			);
			assert(false);
		} catch(DateTimeException) {
			// STRICTだと怒られる。
			assert(true);
		}
		
		// 文字列から生成
		// YYYY-MM-DDThh:mm:ss.MS形式ならDateTimeに変更可能
		with(new DateTime(DateTime.MODE.FLEXIBLE, Text("2010-01-02T03:04:05.678"))) {
			assert(year   == 2010);
			assert(month  == 1);
			assert(day    == 2);
			assert(hour   == 3);
			assert(minute == 4);
			assert(second == 5);
			assert(ms     == 678);
		}
		// 限定的に生成することも可能
		with(new DateTime(DateTime.MODE.FLEXIBLE, Text("2010-12"))) {
			assert(year   == 2010);
			assert(month  == 12);
			assert(day    == 1);
			assert(hour   == 0);
			assert(minute == 0);
			assert(second == 0);
			assert(ms     == 0);
		}
		// FLEXIBLEで生成
		with(new DateTime(DateTime.MODE.FLEXIBLE, Text("2010-12-33"))) {
			assert(year   == 2011);
			assert(month  == 1);
			assert(day    == 2);
		}
		try {
			// STRICTで生成
			with(new DateTime(DateTime.MODE.STRICT, Text("2010-12-33"))) {
				assert(false);
			}
		} catch(DateTimeException) {
			// やっぱり怒られる
			assert(true);
		}
		
		// 文字列化と書式とマスク
		// まずは 2010-01-02T03:04:05.006 で生成
		auto dt4=new DateTime(DateTime.MODE.FLEXIBLE, 2010, 1, 2, 3, 4, 5, 6);
		// ミリ秒が無い
		assert(dt4.toText == Text("2010-01-02T03:04:05"));
		// DateTimeのデフォルト書式にそもそもミリ秒が無い
		assert(dt4.format == Text("[YYYY]-[MM]-[DD]T[hh]:[mm]:[ss]"));
		// 書式設定、ミリ秒を追加。
		dt4.format ~= Text(".[ff]");
		// 追加完了
		assert(dt4.format == Text("[YYYY]-[MM]-[DD]T[hh]:[mm]:[ss].[ff]"));
		// でもなんか変
		assert(dt4.toText == Text("2010-01-02T03:04:05.[ff]"));
		// そもそもデフォルトマスクにミリ秒指定が無い
		assert((dt4.mask & DateTime.MASK.MS) != DateTime.MASK.MS);
		// マスクを設定
		dt4.mask |= DateTime.MASK.MS;
		// 設定完了
		assert((dt4.mask & DateTime.MASK.MS) == DateTime.MASK.MS);
		// 文字列化
		assert(dt4.toText == Text("2010-01-02T03:04:05.006"));
		// 色々文字列化
		// 下二桁の年
		dt4.format = Text("[YY]");
		assert(dt4.toText == Text("10"));
		// 0で埋めない
		dt4.format = Text("[Y]-[M]-[D]T[h]:[m]:[s].[f]");
		assert(dt4.toText == Text("2010-1-2T3:4:5.6"));
		// 曜日
		dt4.mask |= DateTime.MASK.WEEK;
		dt4.format = Text("[@$W]");
		// 2010/1/2は土曜日
		assert(dt4.toText == Text("Sat"));
		assert(dt4.week   == SYSTEMDATETIME.WEEK.SATURDAY);
		// 2010/1/3は日曜日
		dt4.day = 3;
		assert(dt4.toText == Text("Sun"));
		assert(dt4.week   == SYSTEMDATETIME.WEEK.SUNDAY);
		// モード変更, 2010/1/4は月曜日
		dt4.mode = DateTime.MODE.STRICT;
		dt4.day  = 4;
		assert(dt4.toText == Text("Mon")); // STRICTでも曜日は勝手に変わる
		assert(dt4.week   == SYSTEMDATETIME.WEEK.MONDAY);
		// 曜日名の変更, 2010/1/5は火曜日
		dt4.format = Text("[@W]");
		dt4.weeks[SYSTEMDATETIME.WEEK.TUESDAY] = Text("火曜日");
		dt4.day  = 5;
		assert(dt4.toText == Text("火曜日"));
		assert(dt4.week   == SYSTEMDATETIME.WEEK.TUESDAY);
		// 月も数値じゃなくて名前でとってみる
		dt4.format = Text("[@M]");
		assert(dt4.toText == Text("January"));
		
		// それっぽく日曜日
		dt4.day = 3;
		dt4.format = Text("[@$W] [@$M] [D] [hh]:[mm]:[ss] [YYYY]");
		assert(dt4.toText == Text("Sun Jan 3 03:04:05 2010"));
	}
	invariant() {
		if((MASK.YEAR & mask) == MASK.YEAR) {
			assert(SYSTEMDATETIME.isYear(SystemDateTime.year));
		}
		if((MASK.MONTH & mask) == MASK.MONTH) {
			assert(SYSTEMDATETIME.isMonth(SystemDateTime.month));
		}
		if((MASK.DAY & mask) == MASK.DAY) {
			assert(SYSTEMDATETIME.isDay(SystemDateTime.day));
		}
		//if((MASK.WEEK & mask) == MASK.WEEK) {
			assert(SYSTEMDATETIME.isWeek(SystemDateTime.week));
		//}
		if((MASK.HOUR & mask) == MASK.HOUR) {
			assert(SYSTEMDATETIME.isHour(SystemDateTime.hour));
		}
		if((MASK.MINUTE & mask) == MASK.MINUTE) {
			assert(SYSTEMDATETIME.isMinute(SystemDateTime.minute));
		}
		if((MASK.SECOND & mask) == MASK.SECOND) {
			assert(SYSTEMDATETIME.isSecond(SystemDateTime.second));
		}
	}
	
	protected SYSTEMDATETIME SystemDateTime;
	mixin(SMixInStructHiddenOriginal!(SYSTEMDATETIME)(q{SystemDateTime}));

	/***/
	protected const ref Text FormatSingle(in Text FormatText, in Text TargetToken, int Width, int Value) {
		if(Width) {
			Text Data;
			if(GetDigits(Value) >= Width) {
				Data = Text(Value);
				Data = Data[Data.length - Width .. Data.length];
			} else {
				Data = Text("%0.*s", Width, Value);
			}
			return FormatText.replace(TargetToken, Data);//sub(FormatTarget, RegToken, s);
		} else {
			return FormatText.replace(TargetToken, Text(Value));
		}
	}

	///
	private static Text FormatSingle(int Value, int Width, bool ZeroFill) {
		if(Width) {
			Text Data;
			if(GetDigits(Value) >= Width) {
				Data = Text(Value);
				Data = Data[Data.length - Width .. Data.length];
			} else {
				dstring Format;
				if(ZeroFill) {
					Format = "%0.*s";
					Data = Text(Format, Width, Value);
				} else {
					//Format = "%.*s";
					Data = Text(Value).fill(Text.FILL.LEFT, ' ', Width);
				}
				//Data = Text(Format, Width, Value);
			}
			return Data;
		} else {
			return Text(Value);
		}
	}
	///
	private const const(Text) FormatValue(const ref Text Value, out bool FormatFlag) {
		FormatFlag = true;
		DateTimeException e;
		void ExceptionMake(string s) {
			assert(!e);
			if(mode == MODE.STRICT) {
				e = new DateTimeException(Text("fraud " ~ s));
			}
		};
		
		switch(Value.data[$-1]) {
			case 'Y':
				if((MASK.YEAR & mask) == MASK.YEAR) {
					switch(Value.data) {
						case "Y":     return FormatSingle(SystemDateTime.year, 0, false);
						case "YY":    return FormatSingle(SystemDateTime.year, 2, true);
						case "YYYY":  return FormatSingle(SystemDateTime.year, 4, true);
						case "#YY":   return FormatSingle(SystemDateTime.year, 2, false);
						case "#YYYY": return FormatSingle(SystemDateTime.year, 4, false);
						default:
							ExceptionMake("year");
							goto _default;
					}
				}
				break;
			case 'M':
				if((MASK.MONTH & mask) == MASK.MONTH) {
					switch(Value.data) {
						case "M":    return FormatSingle(SystemDateTime.month, 0, false);
						case "MM":   return FormatSingle(SystemDateTime.month, 2, true);
						case "#MM":  return FormatSingle(SystemDateTime.month, 2, false);
						case "@M":   return monthName();
						case "@$MM": return defaultMonths[SystemDateTime.month-1];
						case "@$M":  return defaultMonths[SystemDateTime.month-1][0 .. 3];
						default:
							ExceptionMake("month");
							goto _default;
					}
				}
				break;
			case 'D':
				if((MASK.DAY & mask) == MASK.DAY) {
					switch(Value.data) {
						case "D":   return FormatSingle(SystemDateTime.day, 0, false);
						case "DD":  return FormatSingle(SystemDateTime.day, 2, true);
						case "#DD": return FormatSingle(SystemDateTime.day, 2, false);
						default:
							ExceptionMake("day");
							goto _default;
					}
				}
				break;
			case 'W':
				if((MASK.WEEK & mask) == MASK.WEEK) {
					switch(Value.data) {
						case "W":    return FormatSingle(SystemDateTime.week, 0, false);
						case "@W":   return weekName();
						case "@$WW": return defaultWeeks[SystemDateTime.week];
						case "@$W":  return defaultWeeks[SystemDateTime.week][0 .. 3];
						default:
							ExceptionMake("week");
							goto _default;
					}
				}
				break;
			case 'h':
				if((MASK.HOUR & mask) == MASK.HOUR) {
					switch(Value.data) {
						case "h":    return FormatSingle(SystemDateTime.hour, 0, false);
						case "hh":   return FormatSingle(SystemDateTime.hour, 2, true);
						case "#hh":  return FormatSingle(SystemDateTime.hour, 2, false);
						case "@h":   return FormatSingle(SYSTEMDATETIME.getMeridiemHour(SystemDateTime.hour), 0, false);
						case "@hh":  return FormatSingle(SYSTEMDATETIME.getMeridiemHour(SystemDateTime.hour), 2, true);
						case "@#hh": return FormatSingle(SYSTEMDATETIME.getMeridiemHour(SystemDateTime.hour), 2, false);
						default:
							ExceptionMake("hour");
							goto _default;
					}
				}
				break;
			case 'm':
				if((MASK.MINUTE & mask) == MASK.MINUTE) {
					switch(Value.data) {
						case "m":   return FormatSingle(SystemDateTime.minute, 0, false);
						case "mm":  return FormatSingle(SystemDateTime.minute, 2, true);
						case "#mm": return FormatSingle(SystemDateTime.minute, 2, false);
						default:
							ExceptionMake("minute");
							goto _default;
					}
				}
				break;
			case 's':
				if((MASK.SECOND & mask) == MASK.SECOND) {
					switch(Value.data) {
						case "s":   return FormatSingle(SystemDateTime.second, 0, false);
						case "ss":  return FormatSingle(SystemDateTime.second, 2, true);
						case "#ss": return FormatSingle(SystemDateTime.second, 2, false);
						default:
							ExceptionMake("second");
							goto _default;
					}
				}
				break;
			case 'f':
				if((MASK.MS & mask) == MASK.MS) {
					switch(Value.data) {
						case "f":   return FormatSingle(SystemDateTime.ms, 0, false);
						case "ff":  return FormatSingle(SystemDateTime.ms, 3, true);
						case "#ff": return FormatSingle(SystemDateTime.ms, 3, false);
						default:
							ExceptionMake("ms");
							goto _default;
					}
				}
				break;
			case 'P':
				if((MASK.AM_PM & mask) == MASK.AM_PM) {
					switch(Value.data) {
						case "@AP":  return meridiems[SYSTEMDATETIME.getMeridiem(SystemDateTime.hour)];
						case "@$AP": return defaultMeridiems[SYSTEMDATETIME.getMeridiem(SystemDateTime.hour)];
						default:
							ExceptionMake("AM/PM");
							goto _default;
					}
				}
				break;
			_default:
			default:
		}
		if(mode == MODE.STRICT) {
			throw new DateTimeException(Text("fraud format, %s", Value), e);
		}
		FormatFlag = false;
		return Text.init;
	}
	/**
	History:
		$(HIST_VER 1.120,
			$(HIST_F 書式の大幅変更。)
			$(HIST_F 若干の高速化。)
			$(HIST_P '['を表示可能に。)
		)
		$(HIST_VER 1.110,
			$(HIST_P [@MM], [@Mm], [@WW], [@Ww], [@h], [@hh], [@AP], [@ap]の追加。)
		)
	*/
	protected const ref Text FormatDate(in Text FormatText, MASK Mask, MODE Mode) {
		auto app=appender!(tchar[]);
		auto s=FormatText.data;
		for(auto i=0; i < s.length; i++) {
			if(s[i] == '[') {
				if(i < s.length-1 && s[i+1] == '[') {
					app.put('[');
					i++;
					continue;
				}
				auto Index=FormatText[i+1..FormatText.length].indexOf(']');
				if(Index == -1) {
					if(Mode == MODE.STRICT) {
						throw new DateTimeException(Text("not found ']'"));
					}
					app.put('[');
					continue;
				}
				const Src=FormatText[i .. i+1+Index+1];
				if(Src.length < 3) {
					if(Mode == MODE.STRICT) {
						throw new DateTimeException(Text("not found FormatChar"));
					}
					app.put(Src.data);
					i++;
					continue;
				}
				const Value=Src[1 .. Src.length-1];
				assert(Value.length);
				bool FormatFlag;
				auto Formated=FormatValue(Value, FormatFlag);
				
				if(FormatFlag) {
					app.put(Formated.data);
				} else {
					app.put(Src.data);
				}
				i += Index+1;
			} else {
				app.put(s[i]);
			}
		}
		
		return Text.wrap(app.data);
	}
	debug(time) unittest {
		auto dt=new DateTime(MODE.STRICT, 123, 4, 5, 6, 7, 8, 9);
		dt.mask = DateTime.MASK.ALL;
		dt.format = Text("[Y]@[YY]@[YYYY]@[#YY]@[#YYYY]");
		assert(dt.toText == Text("123@23@0123@23@ 123"));
		dt.format = Text("[M]@[MM]@[#MM]@[@M]@[@$MM]@[@$M]");
		assert(dt.toText == Text("4@04@ 4@April@April@Apr"));
		dt.format = Text("[D]@[DD]@[#DD]");
		assert(dt.toText == Text("5@05@ 5"));
		dt.format = Text("[W]@[@W]@[@$WW]@[@$W]");
		assert(dt.toText == Text("1@Monday@Monday@Mon"));
		dt.format = Text("[h]@[hh]@[#hh]@[@h]@[@hh]@[@#hh]");
		assert(dt.toText == Text("6@06@ 6@6@06@ 6"));
		dt.format = Text("[m]@[mm]@[#mm]");
		assert(dt.toText == Text("7@07@ 7"));
		dt.format = Text("[s]@[ss]@[#ss]");
		assert(dt.toText == Text("8@08@ 8"));
		dt.format = Text("[f]@[ff]@[#ff]");
		assert(dt.toText == Text("9@009@  9"));
		dt.format = Text("[@AP]@[@$AP]");
		assert(dt.toText == Text("AM@AM"));

		dt.year = 2010;
		dt.format = Text("[Y]@[YY]@[YYYY]@[#YY]@[#YYYY]");
		assert(dt.toText == Text("2010@10@2010@10@2010"));
		dt.year = 12345;
		dt.format = Text("[Y]@[YY]@[YYYY]@[#YY]@[#YYYY]");
		assert(dt.toText == Text("12345@45@2345@45@2345"));
	}
	debug(time) unittest {
		//scope d=new DTimer;
		DateTime dt;
		Text t;
		dt = new DateTime(MODE.STRICT, 2010);
		dt.mask   = DateTime.MASK.ALL;
		dt.format = Text("[@h]");
		dt.hour = 0;
		assert(dt.toText == Text("0"));
		dt.hour = 11;
		assert(dt.toText == Text("11"));
		dt.hour = 12;
		assert(dt.toText == Text("0"));
		dt.hour = 23;
		assert(dt.toText == Text("11"));
		dt.format = Text("[@hh]");
		dt.hour = 0;
		assert(dt.toText == Text("00"));
		dt.hour = 11;
		assert(dt.toText == Text("11"));
		dt.hour = 12;
		assert(dt.toText == Text("00"));
		dt.hour = 23;
		assert(dt.toText == Text("11"));
		
		dt.format = Text("[@#hh]");
		dt.hour = 0;
		assert(dt.toText == Text(" 0"));
		dt.format = Text("[@#hh]");
		dt.hour = 10;
		assert(dt.toText == Text("10"));
		
		dt.format = Text("[@$AP]");
		dt.hour = 0;
		assert(dt.toText == Text("AM"));
		dt.hour = 20;
		assert(dt.toText == Text("PM"));

		dt.meridiems[0] = Text("午前");
		dt.meridiems[1] = Text("午後");
		dt.format = Text("[@AP]");
		dt.hour = 0;
		assert(dt.toText == Text("午前"));
		dt.hour = 20;
		assert(dt.toText == Text("午後"));
	}

	/**
	Note:
		取り敢えずべた書き。
	*/
	protected static Throwable TogetherAssignment(ref SYSTEMDATETIME SystemDateTime) {
		/+
		Exception BaseExp=new DateTimeException(Text(`TogetherAssignment`));
		Throwable e = BaseExp;
		+/
		auto tc=THROWABLECHAIN(new DateTimeException(Text(`TogetherAssignment`)));

		// ミリ秒
		if(!SYSTEMDATETIME.isMs(SystemDateTime.ms)) {
			tc.chain(new DateTimeException(Text("ms -> %s", SystemDateTime.ms)));
			
			size_t NextUp=0;
			do {
				SystemDateTime.ms = cast(WORD)(SystemDateTime.ms - (SYSTEMDATETIME.RANGE.MS_MAX+1));
				NextUp++;
			} while(!SYSTEMDATETIME.isMs(SystemDateTime.ms));
			SystemDateTime.second = cast(WORD)(SystemDateTime.second + NextUp);
		}

		// 秒
		if(!SYSTEMDATETIME.isSecond(SystemDateTime.second)) {
			tc.chain(new DateTimeException(Text("second -> %s", SystemDateTime.second)));

			size_t NextUp=0;
			
			do {
				SystemDateTime.second = cast(WORD)(SystemDateTime.second - (SYSTEMDATETIME.RANGE.SECOND_MAX+1));
				NextUp++;
			} while(!SYSTEMDATETIME.isSecond(SystemDateTime.second));
			SystemDateTime.minute = cast(WORD)(SystemDateTime.minute + NextUp);
		}

		// 分
		if(!SYSTEMDATETIME.isMinute(SystemDateTime.minute)) {
			tc.chain(new DateTimeException(Text("minute -> %s", SystemDateTime.minute)));

			size_t NextUp=0;
			
			do {
				SystemDateTime.minute = cast(WORD)(SystemDateTime.minute - (SYSTEMDATETIME.RANGE.MINUTE_MAX+1));
				NextUp++;
			} while(!SYSTEMDATETIME.isMinute(SystemDateTime.minute));
			SystemDateTime.hour = cast(WORD)(SystemDateTime.hour + NextUp);
		}

		// 時間
		if(!SYSTEMDATETIME.isHour(SystemDateTime.hour)) {
			tc.chain(new DateTimeException(Text("hour -> %s", SystemDateTime.hour)));

			size_t NextUp=0;
			
			do {
				SystemDateTime.hour = cast(WORD)(SystemDateTime.hour - (SYSTEMDATETIME.RANGE.HOUR_MAX+1));
				NextUp++;
			} while(!SYSTEMDATETIME.isHour(SystemDateTime.hour));
			SystemDateTime.day = cast(WORD)(SystemDateTime.day + NextUp);
		}

		void Month() {
			// 月
			if(SYSTEMDATETIME.isMonth(SystemDateTime.month)) {
				auto LastDay=SYSTEMDATETIME.getMonthDay(SystemDateTime.year, SystemDateTime.month);
				if(SystemDateTime.day > LastDay) {
					tc.chain(new DateTimeException(Text("month/day -> %s/%s", SystemDateTime.month, SystemDateTime.day)));
					SystemDateTime.day = LastDay;
				}
			} else {
				tc.chain(new DateTimeException(Text("month -> %s", SystemDateTime.month)));
				
				SystemDateTime.year  = cast(WORD)(SystemDateTime.month / 12 + SystemDateTime.year);
				SystemDateTime.month = cast(WORD)(SystemDateTime.month % 12);
				if(!SystemDateTime.month) {
					SystemDateTime.month = 12;
				}

				auto LastDay=SYSTEMDATETIME.getMonthDay(SystemDateTime.year, SystemDateTime.month);
				if(SystemDateTime.day > LastDay) {
					tc.chain(new DateTimeException(Text("month/day -> %s/%s", SystemDateTime.month, SystemDateTime.day)));
					SystemDateTime.day = LastDay;
				}
			}
		}
		
		// 日
		void Day() {
			auto Check = (WORD LastDay) {
				SystemDateTime.day = cast(WORD)(SystemDateTime.day - LastDay);
				auto NextMonth = SYSTEMDATETIME.nextMonth(SystemDateTime.month);
				if(NextMonth != SystemDateTime.month) {
					tc.chain(new DateTimeException(Text("NextMonth != month -> %s != %s", NextMonth, SystemDateTime.month)));

					if(NextMonth == 1) {
						SystemDateTime.year = cast(WORD)(SystemDateTime.year + 1);
					}
					SystemDateTime.month = NextMonth;
				}
			};
			if(SYSTEMDATETIME.isDay(SystemDateTime.day)) {
				auto LastDay=SYSTEMDATETIME.getMonthDay(SystemDateTime.year, SystemDateTime.month);
				if(LastDay < SystemDateTime.day) {
					// 次の月
					tc.chain(new DateTimeException(Text("LastDay < day -> %s < %s", LastDay, SystemDateTime.day)));
					
					Check(LastDay);
				}
			} else {
				tc.chain(new DateTimeException(Text("day -> %s", SystemDateTime.day)));
				
				do {
					Check(SYSTEMDATETIME.getMonthDay(SystemDateTime.year, SystemDateTime.month));
				} while(SystemDateTime.day > SYSTEMDATETIME.getMonthDay(SystemDateTime.year, SystemDateTime.month));
			}
		}

		if(!SYSTEMDATETIME.isYear(SystemDateTime.year)) {
			tc.chain(new DateTimeException(Text("year -> %s", SystemDateTime.year)));
			SystemDateTime.year = 1;
		}
		if(!SYSTEMDATETIME.isMonth(SystemDateTime.month) && !SYSTEMDATETIME.isDay(SystemDateTime.day)) {
			tc.chain(new DateTimeException(Text("month -> %s, day -> %s", SystemDateTime.month, SystemDateTime.day)));
			SystemDateTime.month = SYSTEMDATETIME.RANGE.MONTH_MIN;
			SystemDateTime.day   = SYSTEMDATETIME.RANGE.DAY_MIN;
		}

		if(SYSTEMDATETIME.isDay(SystemDateTime.day)) {
			Month();
			Day();
		} else {
			Day();
			Month();
		}

		/+
		if(BaseExp.next) {
			return BaseExp;
		} else {
			return null;
		}
		+/
		return tc();
	}

	static immutable {
		/**
		標準書式。
		
		History:
			$(HIST_VER 1.101,
				$(HIST_NEW)
			)
		*/
		Text defaultFormat={data: "[YYYY]-[MM]-[DD]T[hh]:[mm]:[ss]"w.dup};
		/**
		標準月名。
		
		History:
			$(HIST_VER 1.110,
				$(HIST_P 短縮形ではない形に変更。)
			)
			$(HIST_VER 1.101,
				$(HIST_NEW)
			)
		*/
		Text[12] defaultMonths=[
			{data: "January"w.dup}, {data: "February"w.dup}, {data: "March"w.dup},     {data: "April"w.dup},   {data: "May"w.dup},      {data: "June"w.dup},
			{data: "July"w.dup},    {data: "August"w.dup},   {data: "September"w.dup}, {data: "October"w.dup}, {data: "November"w.dup}, {data: "December"w.dup}
		];
		/**
		標準曜日名。
		
		History:
			$(HIST_VER 1.110,
				$(HIST_P 短縮形ではない形に変更。)
			)
			$(HIST_VER 1.101,
				$(HIST_NEW)
			)
		*/
		Text[7] defaultWeeks = [{data: "Sunday"w.dup}, {data: "Monday"w.dup}, {data: "Tuesday"w.dup}, {data: "Wednesday"w.dup}, {data: "Thursday"w.dup}, {data: "Friday"w.dup}, {data: "Saturday"w.dup} ];

		/**
		History:
			$(HIST_VER 1.110,
				$(HIST_NEW)
			)
		*/
		Text[2] defaultMeridiems = [{data: "AM"w.dup}, {data: "PM"w.dup}];
	}
	
	/**
	History:
		$(HIST_VER 1.110,
			$(HIST_P months, weeksの初期化処理をinitializeへ。)
		)
		$(HIST_VER 1.101,
			$(HIST_P 初期化処理の定数部分を若干変更。)
		)
	*/
	protected void Initialize(MODE Mode) {
		//format = Text("[YYYY]-[MM]-[DD]T[hh]:[mm]:[ss]");
		//format = defaultFormat.dup;
		mask   = MASK.DATE | MASK.TIME;
		mode   = Mode;
		/+
		//months = Texts("Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec");
		for(auto i=0; i < defaultMonths.length; i++) {
			months[i] = defaultMonths[i].dup;
		}
		//weeks  = Texts("Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat");
		for(auto i=0; i < defaultWeeks.length; i++) {
			weeks[i] = defaultWeeks[i].dup;
		}
		+/
		_initialize();
	}
	/**
	書式、月名と曜日名の初期化。

	formatをdefaultFormatに設定。
	months, weeksにdefaultMonths, defaultWeeksの値を設定。

	History:
		$(HIST_VER 1.110,
			$(HIST_NEW)
	)
	*/
	protected void _initialize() {
		format = defaultFormat.dup;
		
		for(auto i=0; i < defaultMonths.length; i++) {
			months[i] = defaultMonths[i].dup;
		}
		for(auto i=0; i < defaultWeeks.length; i++) {
			weeks[i] = defaultWeeks[i].dup;
		}
		for(auto i=0; i < defaultMeridiems.length; i++) {
			meridiems[i] = defaultMeridiems[i].dup;
		}
	}
	override void initialize() {
		_initialize();
	}

	protected this(MODE Mode, const ref SYSTEMDATETIME SystemDateTime, bool Check) {
		Initialize(Mode);

		if(Check) {
			auto sdt=SystemDateTime;
			auto e=TogetherAssignment(sdt);
			ModeChange(sdt,e);
		} else {
		this.SystemDateTime = SystemDateTime;
			ChangeWeek();
		}
	}
	/**
	現在時刻からDateTimeの生成。

	Params:
		Mode = 日時設定時の挙動。
	*/
	this(MODE Mode=MODE.FLEXIBLE) {
		this(Mode, SYSTEMDATETIME.localTime, false);
	}
	/**
	指定日時からDateTimeの生成。
	
	Params:
		Mode = 日時設定時の挙動。

		SystemDateTime = 設定する日時情報。

	Throws:
		ModeがMODE.STRICTでSystemDateTimeが不正な場合にDateTimeException。
	*/
	this(MODE Mode, const ref SYSTEMDATETIME SystemDateTime) {
		this(Mode, SystemDateTime, true);
	}

	/**
	指定日時からDateTimeの生成。
	
	Params:
		Mode = 日時設定時の挙動。

		FileDateTime = 設定する日時情報。

	Throws:
		ModeがMODE.STRICTでFileDateTimeが不正な場合にDateTimeException。
	*/
	this(MODE Mode, const ref FILEDATETIME FileDateTime) {
		this(Mode, FileDateTime.toSystemDateTime(), true);
	}
	/**
	指定日時からDateTimeの生成。
	
	Params:
		Mode = 日時設定時の挙動。

		Year   = 年。

		Month  = 月。

		Day    = 日。

		Hour   = 時。

		Minute = 分。

		Second = 秒。

		Ms     = ミリ秒。

	Throws:
		ModeがMODE.STRICTで引数が不正な場合にDateTimeException。
	*/
	this(MODE Mode, WORD Year, WORD Month=SYSTEMDATETIME.RANGE.MONTH_MIN, WORD Day=SYSTEMDATETIME.RANGE.DAY_MIN, WORD Hour=SYSTEMDATETIME.RANGE.HOUR_MIN, WORD Minute=SYSTEMDATETIME.RANGE.MINUTE_MIN, WORD Second=SYSTEMDATETIME.RANGE.SECOND_MIN, WORD Ms=SYSTEMDATETIME.RANGE.MS_MIN) {
		SYSTEMDATETIME SystemDateTime;
		with(SystemDateTime) {
			year   = Year;
			month  = Month;
			day    = Day;
			hour   = Hour;
			minute = Minute;
			second = Second;
			ms     = Ms;
		}
		this(Mode, SystemDateTime, true);
	}

	/**
	文字列からDateTimeの生成。
	
	Params:
		Mode = 日時設定時の挙動。

		Value = 日時情報に変換可能な文字列。
		        YYYY-MM-DDThh:mm:ss.MS形式(0埋め不要)で指定。
		        
		        年以外は順々に省略可能。

	Throws:
		Valueを分解できない場合にDateTimeParseException。
		ModeがMODE.STRICTで引数が不正な場合にDateTimeException。

	History:
		$(HIST_VER 1.101,
			$(HIST_F DateTimeParseException追加。)
			$(HIST_B 分割後の代入でひたすら例外。)
		)
	*/
	this(MODE Mode, in Text Value) {
		enforce(Value.length, new DateTimeParseException(Text("Value.length -> 0"), Value));

		Text Date, Time;
		if(auto DateAndTime=Value.split('T')) {
			// 日付と時間。
			enforce(DateAndTime.length == 2, new DateTimeParseException(Text("Value.split('T') -> %s", DateAndTime), Value));
			
			Date = DateAndTime[0];
			Time = DateAndTime[1];
		} else {
			// 日付のみ。
			Date = Value.dup;
		}
		enforce(Date.length, new DateTimeParseException(Text("Date.length -> 0"), Value));
		
		Text Year, Month, Day;
		if(Date.count('-')) {
			auto ymd=Date.split('-');
			Year = ymd[0];
			enforce(ymd.length <= 3, new DateTimeParseException(Text("[YYYY-MM-DD].split('-') <= 3, [YYY-MM-DD] -> %s", ymd), Value));
			if(ymd.length >= 2) {
				Month = ymd[1];
			}
			if(ymd.length == 3) {
				Day = ymd[2];
			}
		} else {
			// 年のみ
			Year = Date;
		}
		// 年は必須
		enforce(Year.length, new DateTimeParseException(Text("Year.length -> 0"), Value));

		Text Hour, Minute, Second, Ms;
		if(Time.length) {
			// 年月日は確定していないとワケ分からん。
			enforce(Year.length && Month.length && Day.length, new DateTimeParseException(Text("Year.length -> %s, Month.length -> %s, Day.length -> %s", Year, Month, Day), Value));
			if(Time.count(':')) {
				auto hms=Time.split(':');
				Hour = hms[0];
				enforce(hms.length <= 3, new DateTimeParseException(Text("[hh:mm-ss.ms].split(':') <= 3, [hh:mm-ss.ms] -> %s", hms), Value));
				if(hms.length >= 2) {
					Minute = hms[1];
				}
				if(hms.length == 3) {
					// ミリ秒はある？
					if(hms[2].count('.')) {
						auto SecondAndMs=hms[2].split('.');
						enforce(SecondAndMs.length == 2, new DateTimeParseException(Text("SecondAndMs.length == 2, SecondAndMs -> %s", SecondAndMs), Value));
						Second = SecondAndMs[0];
						Ms     = SecondAndMs[1];
					} else {
						Second = hms[2];
					}
				}
			} else {
				// 時間のみ
				Hour = Time;
			}
		}

		// 代入
		SYSTEMDATETIME SystemDateTime;
		auto tc=THROWABLECHAIN(new DateTimeException(Text("DateTime Assign, Value -> %s", Value)));
		with(SystemDateTime) {
			bool Result;
			assert(Year.length);
			year   = tc.collect(Year.toNumber!(WORD), Result);

			WORD AssignThis(in Text Value, RANGE Min) {
				WORD result;
				if(Value.length) {
					result = tc.collect(Value.toNumber!(WORD), Result);
				}
				if(result < Min) {
					result = Min;
				}
				return result;
			}
			month  = AssignThis(Month,  RANGE.MONTH_MIN);
			day    = AssignThis(Day,    RANGE.DAY_MIN);
			hour   = AssignThis(Hour,   RANGE.HOUR_MIN);
			minute = AssignThis(Minute, RANGE.MINUTE_MIN);
			second = AssignThis(Second, RANGE.SECOND_MIN);
			ms     = AssignThis(Ms,     RANGE.MS_MIN);
			/+
			if(Month .length) {
				month = tc.collect(Month.toNumber!(WORD), Result);
			}
			if(Day.length) {
				day = tc.collect(Day.toNumber!(WORD), Result);
			}
			if(Hour.length) {
				hour = tc.collect(Hour.toNumber!(WORD), Result);
			}
			if(Minute.length) {
				minute = tc.collect(Minute.toNumber!(WORD), Result);
			}
			if(Second.length) {
				second = tc.collect(Second.toNumber!(WORD), Result);
			}
			if(Ms.length) {
				ms = tc.collect(Ms.toNumber!(WORD), Result);
			}
			+/
		}
		if(tc()) {
			throw tc();
		}
		try {
			this(Mode, SystemDateTime, true);
		} catch(Throwable e) {
			tc.chain(e);
			throw tc();
		}
	}
	debug(time) unittest {
		DateTime dt;
		Text t, f;
		f = Text("[YYYY]-[MM]-[DD]T[hh]:[mm]:[ss].[ff]");
		
		t = Text("2010-02-10T19:30:55.999");
		dt = new DateTime(MODE.STRICT, t);
		dt.format = f;
		dt.mask   = DateTime.MASK.ALL;
		assert(dt.toText == Text("2010-02-10T19:30:55.999"), dt.toString);
		
		t = Text("2010-02-10T19:30:55");
		dt = new DateTime(MODE.STRICT, t);
		dt.format = f;
		dt.mask   = DateTime.MASK.ALL;
		assert(dt.toText == Text("2010-02-10T19:30:55.000"), dt.toString);
		
		t = Text("2010-02-10T19:30");
		dt = new DateTime(MODE.STRICT, t);
		dt.format = f;
		dt.mask   = DateTime.MASK.ALL;
		assert(dt.toText == Text("2010-02-10T19:30:00.000"), dt.toString);
		
		t = Text("2010-02-10T19");
		dt = new DateTime(MODE.STRICT, t);
		dt.format = f;
		dt.mask   = DateTime.MASK.ALL;
		assert(dt.toText == Text("2010-02-10T19:00:00.000"), dt.toString);
		
		t = Text("2010-02-10");
		dt = new DateTime(MODE.STRICT, t);
		dt.format = f;
		dt.mask   = DateTime.MASK.ALL;
		assert(dt.toText == Text("2010-02-10T00:00:00.000"), dt.toString);
		
		t = Text("2010-02");
		dt = new DateTime(MODE.STRICT, t);
		dt.format = f;
		dt.mask   = DateTime.MASK.ALL;
		assert(dt.toText == Text("2010-02-01T00:00:00.000"), dt.toString);
		
		t = Text("2010");
		dt = new DateTime(MODE.STRICT, t);
		dt.format = f;
		dt.mask   = DateTime.MASK.ALL;
		assert(dt.toText == Text("2010-01-01T00:00:00.000"), dt.toString);
	}

	/**
	日時書式。

	$(DEF_LIST_EX info,
		$(DEF_TERM [Y] this.year)
		$(DEF_DESC
			$(PARAGRAPH
				年の桁数を指定しない。
			)
			$(TABLE_DATASEET $(TABLE_BODY
				$(TABLE_ROW $(TABLE_COL_HEAD データ)
					$(TABLE_COL_DATA 12345)
					$(TABLE_COL_DATA 1234)
					$(TABLE_COL_DATA 123)
					$(TABLE_COL_DATA 12)
					$(TABLE_COL_DATA 1)
				)
				$(TABLE_ROW $(TABLE_COL_HEAD 変換後)
					$(TABLE_COL_DATA 12345)
					$(TABLE_COL_DATA 1234)
					$(TABLE_COL_DATA 123)
					$(TABLE_COL_DATA 12)
					$(TABLE_COL_DATA 1)
				)
			)
		))
		$(DEF_TERM [YY], [#YY] this.year)
		$(DEF_DESC
			$(PARAGRAPH
				年の桁数を下二桁に指定。
				10未満の場合、[YY]は0、[#YY]は空白で埋める。
			)
			$(TABLE_DATASEET $(TABLE_BODY
				$(TABLE_ROW $(TABLE_COL_HEAD データ)
					$(TABLE_COL_DATA 12345)
					$(TABLE_COL_DATA 1234)
					$(TABLE_COL_DATA 123)
					$(TABLE_COL_DATA 12)
					$(TABLE_COL_DATA 1)
				)
				$(TABLE_ROW $(TABLE_COL_HEAD 変換後)
					$(TABLE_COL_DATA 45)
					$(TABLE_COL_DATA 34)
					$(TABLE_COL_DATA 23)
					$(TABLE_COL_DATA 12)
					$(TABLE_COL_DATA 01)
				)
			)
		))
		$(DEF_TERM [YYYY], [#YYYY] this.year)
		$(DEF_DESC
			$(PARAGRAPH
				年の桁数を下四桁に指定。
				1000未満の場合、[YYYY]は0、[#YYYY]は空白で埋める。
			)
			$(TABLE_DATASEET $(TABLE_BODY
				$(TABLE_ROW $(TABLE_COL_HEAD データ)
					$(TABLE_COL_DATA 12345)
					$(TABLE_COL_DATA 1234)
					$(TABLE_COL_DATA 123)
					$(TABLE_COL_DATA 12)
					$(TABLE_COL_DATA 1)
				)
				$(TABLE_ROW $(TABLE_COL_HEAD 変換後)
					$(TABLE_COL_DATA 2345)
					$(TABLE_COL_DATA 1234)
					$(TABLE_COL_DATA 0123)
					$(TABLE_COL_DATA 0012)
					$(TABLE_COL_DATA 0001)
				)
			)
		))
		$(DEF_TERM [M] this.month)
		$(DEF_DESC
			$(PARAGRAPH
				月の桁数を指定しない。
			)
			$(TABLE_DATASEET $(TABLE_BODY
				$(TABLE_ROW $(TABLE_COL_HEAD データ)
					$(TABLE_COL_DATA 1)
					$(TABLE_COL_DATA 9)
					$(TABLE_COL_DATA 10)
				)
				$(TABLE_ROW $(TABLE_COL_HEAD 変換後)
					$(TABLE_COL_DATA 1)
					$(TABLE_COL_DATA 9)
					$(TABLE_COL_DATA 10)
				)
			)
		))
		$(DEF_TERM [MM], [#MM] this.month)
		$(DEF_DESC
			$(PARAGRAPH
				月の桁数を二桁に指定。
				10未満の場合、[MM]は0、[#MM]は空白で埋める。
			)
			$(TABLE_DATASEET $(TABLE_BODY
				$(TABLE_ROW $(TABLE_COL_HEAD データ)
					$(TABLE_COL_DATA 1)
					$(TABLE_COL_DATA 9)
					$(TABLE_COL_DATA 10)
				)
				$(TABLE_ROW $(TABLE_COL_HEAD 変換後)
					$(TABLE_COL_DATA 01)
					$(TABLE_COL_DATA 09)
					$(TABLE_COL_DATA 10)
				)
			)
		))
		$(DEF_TERM [@M] this.month)
		$(DEF_DESC
			$(PARAGRAPH
				月の名称を独自の月名で指定。
				DateTime.monthName()を使用してDateTime.monthsから月名を使用する。
			)
		)
		$(DEF_TERM [@$MM] this.month)
		$(DEF_DESC
			$(PARAGRAPH
				月の名称を標準の長い名前で指定。
				DateTime.defaultMonthsを使用する。
			)
		)
		$(DEF_TERM [@$M] this.month)
		$(DEF_DESC
			$(PARAGRAPH
				月の名称を標準の短い名前で指定。
				DateTime.defaultMonthsを使用する。
			)
		)
		$(DEF_TERM [D] this.day)
		$(DEF_DESC
			$(PARAGRAPH
				日の桁数を指定しない。
			)
			$(TABLE_DATASEET $(TABLE_BODY
				$(TABLE_ROW $(TABLE_COL_HEAD データ)
					$(TABLE_COL_DATA 1)
					$(TABLE_COL_DATA 9)
					$(TABLE_COL_DATA 10)
				)
				$(TABLE_ROW $(TABLE_COL_HEAD 変換後)
					$(TABLE_COL_DATA 1)
					$(TABLE_COL_DATA 9)
					$(TABLE_COL_DATA 10)
				)
			)
		))
		$(DEF_TERM [DD], [#DD] this.day)
		$(DEF_DESC
			$(PARAGRAPH
				日の桁数を二桁に指定。
				10未満の場合、[DD]は0、[#DD]は空白で埋める。
			)
			$(TABLE_DATASEET $(TABLE_BODY
				$(TABLE_ROW $(TABLE_COL_HEAD データ)
					$(TABLE_COL_DATA 1)
					$(TABLE_COL_DATA 9)
					$(TABLE_COL_DATA 10)
				)
				$(TABLE_ROW $(TABLE_COL_HEAD 変換後)
					$(TABLE_COL_DATA 01)
					$(TABLE_COL_DATA 09)
					$(TABLE_COL_DATA 10)
				)
			)
		))
		$(DEF_TERM [W] this.week)
		$(DEF_DESC
			$(PARAGRAPH
				曜日を表す番号を指定。
				詳細はSYSTEMDATETIME.WEEKを参照。
			)
			$(TABLE_DATASEET $(TABLE_BODY
				$(TABLE_ROW $(TABLE_COL_HEAD データ)
					$(TABLE_COL_DATA 0)
					$(TABLE_COL_DATA 3)
					$(TABLE_COL_DATA 6)
				)
				$(TABLE_ROW $(TABLE_COL_HEAD 変換後)
					$(TABLE_COL_DATA 0)
					$(TABLE_COL_DATA 3)
					$(TABLE_COL_DATA 6)
				)
			)
		))
		$(DEF_TERM [@W] this.week)
		$(DEF_DESC
			$(PARAGRAPH
				曜日の名称を独自の曜日名で指定。
				DateTime.weekName()を使用してDateTime.weeksから曜日名を使用する。
			)
		)
		$(DEF_TERM [@$WW] this.week)
		$(DEF_DESC
			$(PARAGRAPH
				曜日の名称を標準の長い名前で指定。
				DateTime.defaultWeeksを使用する。
			)
		)
		$(DEF_TERM [@$W] this.week)
		$(DEF_DESC
			$(PARAGRAPH
				曜日の名称を標準の短い名前で指定。
				DateTime.defaultWeeksを使用する。
			)
		)
		$(DEF_TERM [h] this.hour)
		$(DEF_DESC
			$(PARAGRAPH
				時間の桁数を指定しない。
			)
			$(TABLE_DATASEET $(TABLE_BODY
				$(TABLE_ROW $(TABLE_COL_HEAD データ)
					$(TABLE_COL_DATA 1)
					$(TABLE_COL_DATA 9)
					$(TABLE_COL_DATA 10)
				)
				$(TABLE_ROW $(TABLE_COL_HEAD 変換後)
					$(TABLE_COL_DATA 1)
					$(TABLE_COL_DATA 9)
					$(TABLE_COL_DATA 10)
				)
			)
		))
		$(DEF_TERM [hh], [#hh] this.hour)
		$(DEF_DESC
			$(PARAGRAPH
				時間の桁数を二桁に指定。
				10未満の場合、[hh]は0、[#hh]は空白で埋める。
			)
			$(TABLE_DATASEET $(TABLE_BODY
				$(TABLE_ROW $(TABLE_COL_HEAD データ)
					$(TABLE_COL_DATA 1)
					$(TABLE_COL_DATA 9)
					$(TABLE_COL_DATA 10)
				)
				$(TABLE_ROW $(TABLE_COL_HEAD 変換後)
					$(TABLE_COL_DATA 01)
					$(TABLE_COL_DATA 09)
					$(TABLE_COL_DATA 10)
				)
			)
		))
		$(DEF_TERM [@h] this.hour)
		$(DEF_DESC
			$(PARAGRAPH
				十二時間表記で時間の桁数を指定しない。
			)
			$(TABLE_DATASEET $(TABLE_BODY
				$(TABLE_ROW $(TABLE_COL_HEAD データ)
					$(TABLE_COL_DATA 1)
					$(TABLE_COL_DATA 11)
					$(TABLE_COL_DATA 13)
					$(TABLE_COL_DATA 23)
				)
				$(TABLE_ROW $(TABLE_COL_HEAD 変換後)
					$(TABLE_COL_DATA 1)
					$(TABLE_COL_DATA 11)
					$(TABLE_COL_DATA 1)
					$(TABLE_COL_DATA 11)
				)
			)
		))
		$(DEF_TERM [@hh], [@#hh] this.hour)
		$(DEF_DESC
			$(PARAGRAPH
				十二時間表記で時間の桁数を二桁に指定。
				10未満の場合、[@hh]は0、[@#hh]は空白で埋める。
			)
			$(TABLE_DATASEET $(TABLE_BODY
				$(TABLE_ROW $(TABLE_COL_HEAD データ)
					$(TABLE_COL_DATA 1)
					$(TABLE_COL_DATA 11)
					$(TABLE_COL_DATA 13)
					$(TABLE_COL_DATA 23)
				)
				$(TABLE_ROW $(TABLE_COL_HEAD 変換後)
					$(TABLE_COL_DATA 01)
					$(TABLE_COL_DATA 11)
					$(TABLE_COL_DATA 01)
					$(TABLE_COL_DATA 11)
				)
			)
		))
		$(DEF_TERM [m] this.minute)
		$(DEF_DESC
			$(PARAGRAPH
				分の桁数を指定しない。
			)
			$(TABLE_DATASEET $(TABLE_BODY
				$(TABLE_ROW $(TABLE_COL_HEAD データ)
					$(TABLE_COL_DATA 1)
					$(TABLE_COL_DATA 9)
					$(TABLE_COL_DATA 10)
				)
				$(TABLE_ROW $(TABLE_COL_HEAD 変換後)
					$(TABLE_COL_DATA 1)
					$(TABLE_COL_DATA 9)
					$(TABLE_COL_DATA 10)
				)
			)
		))
		$(DEF_TERM [mm], [#mm] this.minute)
		$(DEF_DESC
			$(PARAGRAPH
				分の桁数を二桁に指定。
				10未満の場合、[mm]は0、[#mm]は空白で埋める。
			)
			$(TABLE_DATASEET $(TABLE_BODY
				$(TABLE_ROW $(TABLE_COL_HEAD データ)
					$(TABLE_COL_DATA 1)
					$(TABLE_COL_DATA 9)
					$(TABLE_COL_DATA 10)
				)
				$(TABLE_ROW $(TABLE_COL_HEAD 変換後)
					$(TABLE_COL_DATA 01)
					$(TABLE_COL_DATA 09)
					$(TABLE_COL_DATA 10)
				)
			)
		))
		$(DEF_TERM [s] this.second)
		$(DEF_DESC
			$(PARAGRAPH
				分の桁数を指定しない。
			)
			$(TABLE_DATASEET $(TABLE_BODY
				$(TABLE_ROW $(TABLE_COL_HEAD データ)
					$(TABLE_COL_DATA 1)
					$(TABLE_COL_DATA 9)
					$(TABLE_COL_DATA 10)
				)
				$(TABLE_ROW $(TABLE_COL_HEAD 変換後)
					$(TABLE_COL_DATA 1)
					$(TABLE_COL_DATA 9)
					$(TABLE_COL_DATA 10)
				)
			)
		))
		$(DEF_TERM [ss], [#ss] this.second)
		$(DEF_DESC
			$(PARAGRAPH
				分の桁数を二桁に指定。
				10未満の場合、[ss]は0、[#ss]は空白で埋める。
			)
			$(TABLE_DATASEET $(TABLE_BODY
				$(TABLE_ROW $(TABLE_COL_HEAD データ)
					$(TABLE_COL_DATA 1)
					$(TABLE_COL_DATA 9)
					$(TABLE_COL_DATA 10)
				)
				$(TABLE_ROW $(TABLE_COL_HEAD 変換後)
					$(TABLE_COL_DATA 01)
					$(TABLE_COL_DATA 09)
					$(TABLE_COL_DATA 10)
				)
			)
		))
		$(DEF_TERM [f] this.ms)
		$(DEF_DESC
			$(PARAGRAPH
				ミリ秒の桁数を指定しない。
			)
			$(TABLE_DATASEET $(TABLE_BODY
				$(TABLE_ROW $(TABLE_COL_HEAD データ)
					$(TABLE_COL_DATA 123)
					$(TABLE_COL_DATA 12)
					$(TABLE_COL_DATA 1)
				)
				$(TABLE_ROW $(TABLE_COL_HEAD 変換後)
					$(TABLE_COL_DATA 123)
					$(TABLE_COL_DATA 12)
					$(TABLE_COL_DATA 1)
				)
			)
		))
		$(DEF_TERM [ff], [#ff] this.ms)
		$(DEF_DESC
			$(PARAGRAPH
				ミリ秒の桁数を三桁に指定。
				100未満の場合、[ff]は0、[#ff]は空白で埋める。
			)
			$(TABLE_DATASEET $(TABLE_BODY
				$(TABLE_ROW $(TABLE_COL_HEAD データ)
					$(TABLE_COL_DATA 123)
					$(TABLE_COL_DATA 12)
					$(TABLE_COL_DATA 1)
				)
				$(TABLE_ROW $(TABLE_COL_HEAD 変換後)
					$(TABLE_COL_DATA 123)
					$(TABLE_COL_DATA 012)
					$(TABLE_COL_DATA 001)
				)
			)
		))
		$(DEF_TERM [@AP] this.hour)
		$(DEF_DESC
			$(PARAGRAPH
				午前午後を独自の名前で指定。
				DateTime.hourとSYSTEMDATETIME.getMeridiemを使用してDateTime.meridiemsから午前午後を使用する。
			)
		)
		$(DEF_TERM [@$AP] this.hour)
		$(DEF_DESC
			$(PARAGRAPH
				午前午後を標準の名前で指定。
				DateTime.defaultMeridiemsを使用する。
			)
			$(TABLE_DATASEET $(TABLE_BODY
				$(TABLE_ROW $(TABLE_COL_HEAD データ)
					$(TABLE_COL_DATA 0)
					$(TABLE_COL_DATA 11)
					$(TABLE_COL_DATA 12)
					$(TABLE_COL_DATA 23)
				)
				$(TABLE_ROW $(TABLE_COL_HEAD 変換後)
					$(TABLE_COL_DATA AM)
					$(TABLE_COL_DATA AM)
					$(TABLE_COL_DATA PM)
					$(TABLE_COL_DATA PM)
				)
			))
		)
	)

	History:
		$(HIST_VER 1.120,
			$(HIST_F 書式内容の変更。)
		)
		$(HIST_VER 1.110,
			$(HIST_P 月と曜日の標準書式追加。)
			$(HIST_P 午前午後の書式と標準書式追加。)
			$(HIST_B [ms]のドキュメントが[MS]と同じ。)
		)
	*/
	Text format;
	/// 月の文字列。
	Text[12] months;
	/// 曜日の文字列。
	Text[7]  weeks;
	/**
	AM/PM。

	History:
		$(HIST_VER 1.110,
			$(HIST_NEW)
		)
	*/
	Text[2] meridiems;

	debug(time) unittest {
		auto dt=new DateTime(MODE.STRICT, 1000, 1, 1);
		dt.mask = DateTime.MASK.AM_PM;
		dt.hour = 0;
	}

	/**
	書式。

	History:
		$(HIST_VER 1.110,
			$(HIST_NEW)
		)
	*/
	enum FORMAT {
		DEFAULT, /// 
	}
	/**
	書式の一括変更。

	DateTime.formatと違い、全ての書式を変更する。

	Params:
		Format = 書式。

	History:
		$(HIST_VER 1.110,
			$(HIST_NEW)
		)
	*/
	void changeFormat(FORMAT Format) {
		with(FORMAT) final switch(Format) {
			case DEFAULT:
				Initialize(mode);
				break;
		}
	}
	
	/**
	日時情報から現在の月名を取得。
	*/
	const const(Text) monthName() {
		return months[SystemDateTime.month-1];
	}
	/**
	日時情報から現在の曜日名を取得。
	*/
	const const(Text) weekName() {
		return weeks[SystemDateTime.week];
	}

	/**
	文字列変換時の変換マスク。

	該当しない箇所は文字列変換されない。

	History:
		$(HIST_VER 1.110,
			$(HIST_S AM_PM追加。)
		)
		$(HIST_VER 1.101,
			$(HIST_O ドキュメント内の文言をマスクに統一。)
		)
	*/
	enum MASK {
		YEAR   = 0b_000000001, /// 年。
		MONTH  = 0b_000000010, /// 月。
		DAY    = 0b_000000100, /// 日。
		WEEK   = 0b_000001000, /// 曜日。
		HOUR   = 0b_000010000, /// 時。
		MINUTE = 0b_000100000, /// 分。
		SECOND = 0b_001000000, /// 秒。
		MS     = 0b_010000000, /// ミリ秒。
		AM_PM  = 0b_100000000, /// AM/PM。

		DATE   = YEAR | MONTH | DAY,     /// 年・月・日。
		DATES  = DATE | WEEK,            /// 年・月・日・曜日。
		TIME   = HOUR | MINUTE | SECOND, /// 時・分・秒。
		TIMES  = TIME | MS,              /// 時・分・秒・ミリ秒。

		ALL    = DATES | TIMES | AM_PM, /// 全て。
	}
	/**
	変換マスク。
	*/
	MASK mask;

	/**
	日時設定時の動作。
	*/
	enum MODE {
		FLEXIBLE, /// あふれる場合は次に進めたりする。
		STRICT,   /// 問答無用で例外。
	}
	/// 挙動
	MODE mode;

	private void ModeChange(string file=__FILE__, int line=__LINE__)(const ref SYSTEMDATETIME sdt, Throwable e) {
		if(mode == MODE.FLEXIBLE) {
			SystemDateTime = sdt;
		} else {
			assert(mode == MODE.STRICT);
			if(e) {
				throw e;
			}
			SystemDateTime = sdt;
		}
		ChangeWeek();
	}

	/***/
	protected void ChangeWeek() {
		SystemDateTime.week = SYSTEMDATETIME.getWeekDay(SystemDateTime.year, SystemDateTime.month, SystemDateTime.day);
	}

	/**
	日時情報から現在の曜日を取得。
	*/
	@property const SYSTEMDATETIME.WEEK week() {
		return cast(SYSTEMDATETIME.WEEK)SystemDateTime.week;
	}

	private static pure nothrow string Property(string Method, string InStatement=null) {
		return `
			mixin(SMixInClassGet!(WORD)("` ~ Method ~ `", q{SystemDateTime.` ~ Method ~ `}, false));
		
			@property WORD ` ~ Method ~ `(WORD Value) {
				` ~ (InStatement.length ? InStatement: "") ~ `

				auto sdt=SystemDateTime;
				sdt.` ~ Method ~ ` = Value;
				
				auto e=TogetherAssignment(sdt);
				ModeChange(sdt, e);

				return SystemDateTime.` ~ Method ~ `;
			}
		`;
	}

	mixin(Property("year", q{enforce(SYSTEMDATETIME.isYear(Value), new DateTimeException(Text("year -> %s", Value)));}));
	mixin(Property("month"));
	mixin(Property("day"));
	mixin(Property("hour"));
	mixin(Property("minute"));
	mixin(Property("second"));
	mixin(Property("ms"));
	debug(time) unittest {
		auto t=new DateTime(MODE.STRICT, 2000,1,2,3,4,5);
		assert(t.second = 10 == 10);
	}


	/**
	文字列変換

	maskとformatを元に文字列変換。

	Returns:
		maskがMASK.DATE | MASK.TIMES、formatが[YYYY]-[MM]-[DD]T[hh]:[mm]:[ss].[ff]ならSYSTEMDATETIME.toTextを呼び出し。
		それ以外はDateTimeがちまちま変換。
	
	History:
		$(HIST_VER 1.101,
			$(HIST_B SYSTEMDATETIME.toTextは曜日を含まないのでmaskがMASK.DATE | MASK.TIMESの時にSYSTEMDATETIME.toTextと等価)
		)
		$(HIST_VER 1.100,
			$(HIST_P maskがMASK.ALLであればSYSTEMDATETIME.toTextと等価。)
		)
	*/
	override const Text toText() {
		if(mask == (MASK.DATE | MASK.TIMES) && format == Text("[YYYY]-[MM]-[DD]T[hh]:[mm]:[ss].[ff]")) {
			return SystemDateTime.toText;
		} else {
			return FormatDate(format, mask, mode);
		}
	}
	version(ITEXT) {
		mixin MixInITextToString!(DateTime);
	} else {
		override string toString() {
			return toText.str;
		}
	}

	/**
	閏年判定。

	SYSTEMDATETIME.isLeapYearを内部で呼び出し。
	
	History:
		$(HIST_VER 1.110,
			$(HIST_NEW)
		)
	*/
	const bool isLeapYear() {
		return SYSTEMDATETIME.isLeapYear(year);
	}
}

/***/
struct TIMESTAMP {
	///
	enum FLAG {
		NONE   = 0,
		CREATE = 0b_0001,
		ACCESS = 0b_0010,
		UPDATE = 0b_0100,

		ALL    = CREATE | ACCESS | UPDATE,
	}
	///
	FLAG flags=FLAG.ALL;
	
	DateTime create; /// 作成日時。
	DateTime access; /// アクセス日時。
	DateTime update; /// 更新日時。

	///
	const Text toText() {
		return Text(
			"Create -> %s, Access -> %s, Update -> %s, flags -> %03b",
			create, access, update, flags
		);
	}
}

debug {
import std.perf;
	scope class DTimer: PerformanceCounter {
		this() {
			start();
		}

		~this() {
			stop();
			wl("Timer:%s", microseconds);
		}
	}
}

