﻿/**

*/
module etc.negui.window.window;

debug import std.stdio: wl = writefln;
debug(window) void main() {}

import win32.windows;

//import nemuxi.base;
public import etc.negui.system.base;
public import etc.negui.negui;
import etc.negui.window.menu.menu;

///
class WindowException: NeGuiException {
	mixin MixInNeGuiException;
}

///
class Window: NeGui {

	/**
	History:
		$(HIST_VER 1.100,
			$(HIST_NEW)
		)
	*/
	static Window desktop() {
		return new Window(GetDesktopWindow());
	}

	protected override this(HWND hWnd) {
		super(hWnd);
	}

	/**
	History:
		$(HIST_VER 1.022,
			$(HIST_NEW)
		)
	*/
	protected this(ref NEGUIINFO NeGuiInfo) {
		NeGuiInfo.eventLoop =  true;
		NeGuiInfo.exStyle   |= WS_EX_CONTROLPARENT;
		NeGuiInfo.style     |= WS_POPUP | WS_CLIPCHILDREN;
		
		super(NeGuiInfo);
	}
	
	mixin(SMixInItemExStyleGetSet("contextHelp", q{WS_EX_CONTEXTHELP}));
	mixin(SMixInItemExStyleGetSet("taskBar", q{WS_EX_APPWINDOW}));
	mixin(SMixInItemExStyleGetSet("tab", q{WS_EX_CONTROLPARENT}, false));
	mixin(SMixInItemExStyleGetSet("toolWindow", q{WS_EX_TOOLWINDOW}));
	
	mixin(SMixInItemStyleGetSet("caption", q{WS_CAPTION}));
	mixin(SMixInItemStyleGetSet("clipChildren", q{WS_CLIPCHILDREN}));
	mixin(SMixInItemStyleGetSet("systemMenu", q{WS_SYSMENU}));

	mixin(SMixInItemStyleGetSet("frameThick", q{WS_THICKFRAME}));
	mixin(SMixInItemStyleGetSet("frameSize", q{WS_SIZEBOX}));
	
	mixin(SMixInItemStyleGetSet("maxButton", q{WS_MAXIMIZEBOX}));
	mixin(SMixInItemStyleGetSet("minButton", q{WS_MINIMIZEBOX}));
	

	/**
	*/
	const bool isSmall() {
		return cast(bool)IsIconic(hWnd);
	}
	/**
	*/
	bool openSmall() {
		return cast(bool)OpenIcon(hWnd);
	}
	/**
	*/
	bool toSmall() {
		return cast(bool)CloseWindow(hWnd);
	}
	
	/**
	*/
	const bool isZoom() {
		return cast(bool)IsZoomed(hWnd);
	}

	/**
	*/
	Window toActive() {
		auto hOldActiveWnd=SetActiveWindow(hWnd);
		if(auto OldGui=NeGui.getGuiObject(hOldActiveWnd)) {
			return cast(Window)OldGui;
		} else {
			return new OtherWindow(hOldActiveWnd);
		}
	}
	

	/**
	*/
	static enum CENTER {
		DESKTOP,  /// 
		WORKAREA, /// 
		WINDOW,   /// 
	}
	/**
	History:
		$(HIST_VER 1.021,
			$(HIST_S 引数属性変更。)
		)
	*/
	bool toCenter(CENTER Center, in NeGui OwnerWindow=null)
	in {
		if(Center == CENTER.WINDOW) {
			assert(OwnerWindow.isAlive);
		}
	}
	body {
		auto WindowRect=itemRect();
		
		with(CENTER) final switch(Center) {
			case DESKTOP: {
				RECT DeskTop=void;
				//GetWindowRect(GetDesktopWindow(), &DeskTop);
				DeskTop = desktop.itemRect;
				return super.move(
					DeskTop.right  / 2 - (WindowRect.right  - WindowRect.left) / 2,
					DeskTop.bottom / 2 - (WindowRect.bottom - WindowRect.top)  / 2,
					WindowRect.right  - WindowRect.left,
					WindowRect.bottom - WindowRect.top,
					true
				);
			}
			// あわせねば
			case WORKAREA: {
				RECT WorkArea=void;
				SystemParametersInfo(SPI_GETWORKAREA, 0, &WorkArea, 0);
				return super.move(
					WorkArea.left + (WorkArea.right  - WorkArea.left) / 2 - (WindowRect.right  - WindowRect.left) / 2,
					WorkArea.top  + (WorkArea.bottom - WorkArea.top)  / 2 - (WindowRect.bottom - WindowRect.top)  / 2,
					WindowRect.right  - WindowRect.left,
					WindowRect.bottom - WindowRect.top,
					true
				);
			}
			case WINDOW: {
				auto OwnerRect=OwnerWindow.itemRect();
				
				return super.move(
					OwnerRect.left + (OwnerRect.right  - OwnerRect.left) / 2 - (WindowRect.right  - WindowRect.left) / 2,
					OwnerRect.top  + (OwnerRect.bottom - OwnerRect.top)  / 2 - (WindowRect.bottom - WindowRect.top)  / 2,
					WindowRect.right  - WindowRect.left,
					WindowRect.bottom - WindowRect.top,
					true
				);
			}
			//default:
			//	assert(false);
		}
	}
	/**
	クライアント領域を指定サイズに変更。

	Params:
		Width = 横幅。
		Hwight = 縦幅。

	Returns:
		最終的なサイズ変更が成功すればtrue、失敗すればfalse。

	Throws:
		初っ端から失敗すればWindowException。
	*/
	bool adjustSize(in int Width, in int Height) {
		RECT Rect;
		Rect.right  = Width;
		Rect.bottom = Height;
		if(
			AdjustWindowRectEx(
				&Rect,
				this.getItemInfo(GWL.STYLE),
				GetMenu(hWnd) ? true: false,
				this.getItemInfo(GWL.EXSTYLE)
			)
		) {
			auto ItemRect=itemRect();
			return this.move(
				ItemRect.left, ItemRect.top,
				Rect.right - Rect.left, Rect.bottom - Rect.top,
				true
			);
		}
		
		throw new WindowException(Text("AdjustWindowRectEx"));
	}
	/**
	History:
		$(HIST_VER 1.100,
			$(HIST_S 引数をポインタから参照へ。)
		)
	*/
	bool adjustSize(const ref SIZE Size) {
		return adjustSize(Size.cx, Size.cy);
	}

	/**
	*/
	static Window find(in Text ClassName, in Text TitleName) {
		wchar* pTitle  = TitleName.length ? TitleName.ptr: null;
	
		if(auto hWnd=FindWindow(ClassName.ptr, pTitle)) {
			if(auto gui=cast(Window)NeGui.getGuiObject(hWnd)) {
				return gui;
			} else {
				return new OtherWindow(hWnd);
			}
		} else {
			return null;
		}
	}
	/**
	*/
	static Window find(in Text ClassName) {
		return Window.find(ClassName, Text.init);
	}

	mixin(SMixInItemExStyleGetSet("alphaStyle", q{WS_EX_LAYERED}));
	/**
	History:
		$(HIST_VER 1.070,
			$(HIST_NEW)
		)
	*/
	enum ALPHA_TYPE {
		COLOR = LWA_COLORKEY, /// 透明色として crKey を使います。
		BLEND = LWA_ALPHA,    /// bAlpha を使って、レイヤードウィンドウの不透明度を決定します。
	}
	/**
	History:
		$(HIST_VER 1.070,
			$(HIST_NEW)
		)
	*/
	bool alpha(in COLOR Color, ubyte Blend, ALPHA_TYPE AlphaType) {
		auto NowAlphaStyle=alphaStyle;
		bool AlphaFlag=true;
		/// 
		if((AlphaType & ALPHA_TYPE.BLEND) == ALPHA_TYPE.BLEND) {
			if(Blend == 255 && NowAlphaStyle) {
				AlphaFlag = AlphaType != ALPHA_TYPE.BLEND;
			}
		}
		if(!NowAlphaStyle && AlphaFlag) {
			alphaStyle = true;
		}
		if(AlphaFlag) {
			return cast(bool)SetLayeredWindowAttributes(hWnd, Color.code, Blend, AlphaType);
		} else {
			alphaStyle = false;
			return false;
		}
	}
	/**
	History:
		$(HIST_VER 1.070,
			$(HIST_NEW)
		)
	*/
	bool alphaColor(in COLOR Color) {
		return alpha(Color, 0, ALPHA_TYPE.COLOR);
	}
	/**
	History:
		$(HIST_VER 1.070,
			$(HIST_NEW)
		)
	*/
	bool alphaBlend(ubyte Blend) {
		return alpha(COLOR.init, Blend, ALPHA_TYPE.BLEND);
	}

	///
	enum ANIMATION {
		SLIDE             = AW_SLIDE,        /// スライドアニメーションを使います。既定値では、ロールアニメーションを使います。AW_CENTER フラグを同時にセットすると、このフラグは無視されます。
		ACTIVATE          = AW_ACTIVATE,     /// ウィンドウをアクティブにします。AW_HIDE フラグと同時にセットしないでください。
		BLEND             = AW_BLEND,        /// フェードイン効果を使います。dwTime パラメータは、ウィンドウを表示するときに使うアニメーションのフレーム数を示します。$(DDOC_COMMENT [日本語版の誤訳？]  AW_HIDEフラグと同時にセットしないでください)。
		HIDE              = AW_HIDE,         /// ウィンドウを隠します。既定値では、ウィンドウを表示します。
		CENTER            = AW_CENTER,       /// AW_HIDE フラグをセットしたときは、ウィンドウを中心に向かって折り畳むようにして隠します。また、AW_HIDE フラグをセットしないときは、ウィンドウを中心から徐々に広げるように表示します。
		HORIZON_POSITIVE  = AW_HOR_POSITIVE, /// ウィンドウを左から右に向かってアニメートすることによって、表示したり隠したりします。このフラグは、ロールアニメーションにも、スライドアニメーションにも使えます。AW_CENTER フラグと同時にセットすると、このフラグは無視されます。
		HORIZON_NEGATIVE  = AW_HOR_NEGATIVE, /// ウィンドウを右から左に向かってアニメートすることによって、表示したり隠したりします。このフラグは、ロールアニメーションにも、スライドアニメーションにも使えます。AW_CENTER フラグと同時にセットすると、このフラグは無視されます。
		VERTICAL_POSITIVE = AW_VER_POSITIVE, /// ウィンドウを上から下に向かってアニメートすることによって、表示したり隠したりします。ロールアニメーションにも、スライドアニメーションにも使えます。AW_CENTER フラグと同時にセットすると、このフラグは無視されます。
		VERTICAL_NEGATIVE = AW_VER_NEGATIVE, /// ウィンドウを下から上に向かってアニメートすることによって、表示したり隠したりします。ロールアニメーションにも、スライドアニメーションにも使えます。AW_CENTER フラグと同時にセットすると、このフラグは無視されます。
	}
	/**
	Params:
		Animations = アニメーション方法。

		TimeMs = アニメーション時間。ミリ秒指定。

	Returns:
		成功の真偽値。
		詳細はERRを参照。
	
	History:
		$(HIST_VER 1.120,
			$(HIST_NEW)
		)
	*/
	bool animation(ANIMATION Animations, size_t TimeMs=200) {
		return cast(bool)AnimateWindow(hWnd, TimeMs, Animations);
	}
}

/***/
final class OtherWindow: Window {
	this(HWND hWnd) {
		super(hWnd);
	}
}

Window GetParentWindow(NeGui base) {
	if(cast(Window)base) {
		return cast(Window)base;
	}
	auto gui=base;
	
	while(!cast(Window)gui.parent) {
		gui = gui.parent;
	}

	return cast(Window)gui.parent;
}

